@@ -45,6 +45,7 @@ from datalad.support.exceptions import CommandError
 from datalad.utils import (
     Path,
     ensure_list,
+    todo_interface_for_extensions,
 )
 
 from datalad.distribution.dataset import (
@@ -255,7 +256,7 @@ class Push(Interface):
         matched_anything = False
         for dspath, dsrecords in ds_spec:
             matched_anything = True
-            lgr.debug('Attempt push of Dataset at %s', dspath)
+            lgr.debug('Pushing Dataset at %s', dspath)
             pbars = {}
             yield from _push(
                 dspath, dsrecords, to, data, force, jobs, res_kwargs.copy(), pbars,
@@ -336,8 +337,6 @@ def _datasets_since_(dataset, since, paths, recursive, recursion_limit):
             annex=None,
             recursive=recursive,
             recursion_limit=recursion_limit,
-            # make it as fast as possible
-            eval_file_type=False,
             # TODO?: expose order as an option for diff and push
             # since in some cases breadth-first would be sufficient
             # and result in "taking action faster"
@@ -401,6 +400,21 @@ def _datasets_since_(dataset, since, paths, recursive, recursion_limit):
         yield (cur_ds, ds_res)
 
 
+@todo_interface_for_extensions
+def _transfer_data(repo, ds, target, content, data, force, jobs, res_kwargs,
+                   got_path_arg):
+    yield from _push_data(
+        ds,
+        target,
+        content,
+        data,
+        force,
+        jobs,
+        res_kwargs.copy(),
+        got_path_arg=got_path_arg,
+    )
+
+
 def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
           got_path_arg=False):
     force_git_push = force in ('all', 'gitpush')
@@ -593,7 +607,8 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
             log_progress(
                 lgr.info, pbar_id, "Transfer data",
                 label="Transfer data to '{}'".format(target), update=2, total=4)
-            yield from _push_data(
+            yield from _transfer_data(
+                repo,
                 ds,
                 target,
                 content,
@@ -633,7 +648,7 @@ def _push(dspath, content, target, data, force, jobs, res_kwargs, pbars,
         # server-side git-annex branch updates (and git-annex does
         # not trigger the hook on copy), but we know we have
         # full access via the push url -- we have just used it to copy.
-        lgr.debug("Fetch 'git-annex' branch updates from '%s'", target)
+        lgr.debug("Fetching 'git-annex' branch updates from '%s'", target)
         fetch_cmd = ['fetch', target, 'git-annex']
         pushurl = repo.config.get(
             'remote.{}.pushurl'.format(target), None)
@@ -834,7 +849,7 @@ def _push_data(ds, target, content, data, force, jobs, res_kwargs,
         # if we force, we do not trust local knowledge and do the checks
         cmd.append('--fast')
 
-    lgr.debug("Push data from %s to '%s'", ds, target)
+    lgr.debug("Pushing data from %s to '%s'", ds, target)
 
     # input has type=dataset, but now it is about files
     res_kwargs.pop('type', None)
