@@ -12,48 +12,48 @@
 import logging
 import os.path as op
 import stat
-
 from unittest.mock import patch
 
-from datalad.config import ConfigManager
+import pytest
+
 from datalad import consts
 from datalad.api import (
     clone,
     create,
     remove,
 )
-from datalad.utils import (
-    chpwd,
-    get_home_envvars,
-    Path,
-    on_windows,
-    rmtree
+from datalad.cmd import GitWitlessRunner
+from datalad.cmd import WitlessRunner as Runner
+from datalad.config import ConfigManager
+from datalad.core.distributed.clone import (
+    _get_installationpath_from_url,
+    decode_source_spec,
 )
+from datalad.distribution.dataset import Dataset
+from datalad.support.annexrepo import AnnexRepo
 from datalad.support.exceptions import IncompleteResultsError
 from datalad.support.external_versions import external_versions
 from datalad.support.gitrepo import GitRepo
-from datalad.support.annexrepo import AnnexRepo
-from datalad.cmd import (
-    GitWitlessRunner,
-    WitlessRunner as Runner,
-)
-from datalad.tests.utils import (
+from datalad.support.network import get_local_file_url
+from datalad.tests.utils_pytest import (
+    DEFAULT_BRANCH,
+    DEFAULT_REMOTE,
+    SkipTest,
     assert_false,
     assert_in,
     assert_in_results,
-    assert_not_is_instance,
     assert_message,
     assert_not_in,
+    assert_not_is_instance,
     assert_raises,
     assert_repo_status,
     assert_result_count,
     assert_result_values_equal,
     assert_status,
     create_tree,
-    DEFAULT_BRANCH,
-    DEFAULT_REMOTE,
     eq_,
     get_datasets_topdir,
+    has_symlink_capability,
     integration,
     known_failure,
     known_failure_githubci_win,
@@ -63,11 +63,10 @@ from datalad.tests.utils import (
     nok_,
     ok_,
     ok_file_has_content,
-    has_symlink_capability,
     ok_startswith,
     patch_config,
-    set_date,
     serve_path_via_http,
+    set_date,
     skip_if_adjusted_branch,
     skip_if_no_network,
     skip_if_on_windows,
@@ -77,14 +76,14 @@ from datalad.tests.utils import (
     with_sameas_remote,
     with_tempfile,
     with_tree,
-    SkipTest,
 )
-from datalad.support.network import get_local_file_url
-from datalad.core.distributed.clone import (
-    _get_installationpath_from_url,
-    decode_source_spec,
+from datalad.utils import (
+    Path,
+    chpwd,
+    get_home_envvars,
+    on_windows,
+    rmtree,
 )
-from datalad.distribution.dataset import Dataset
 
 # this is the dataset ID of our test dataset in the main datalad RIA store
 datalad_store_testds_id = '76b6ca66-36b1-11ea-a2e6-f0d5bf7b5561'
@@ -93,7 +92,7 @@ datalad_store_testds_id = '76b6ca66-36b1-11ea-a2e6-f0d5bf7b5561'
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_invalid_args(path, otherpath, alienpath):
+def test_invalid_args(path=None, otherpath=None, alienpath=None):
     # source == path
     assert_raises(ValueError, clone, 'Zoidberg', path='Zoidberg')
     assert_raises(ValueError, clone, 'ssh://mars/Zoidberg', path='ssh://mars/Zoidberg')
@@ -126,7 +125,7 @@ def test_invalid_args(path, otherpath, alienpath):
 @skip_if_no_network
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_clone_crcns(tdir, ds_path):
+def test_clone_crcns(tdir=None, ds_path=None):
     with chpwd(tdir):
         res = clone('///', path="all-nonrecursive", on_failure='ignore',
                     result_xfm=None, return_type='list')
@@ -143,7 +142,7 @@ def test_clone_crcns(tdir, ds_path):
 @integration
 @skip_if_no_network
 @with_tree(tree={'sub': {}})
-def test_clone_datasets_root(tdir):
+def test_clone_datasets_root(tdir=None):
     tdir = Path(tdir)
     with chpwd(tdir):
         ds = clone("///")
@@ -217,7 +216,7 @@ def check_clone_simple_local(src, path):
 
 @with_tempfile(mkdir=True)
 @serve_path_via_http
-def test_clone_simple_local(src, url):
+def test_clone_simple_local(src=None, url=None):
     srcobj = Path(src)
     gitds = Dataset(srcobj / 'git').create(annex=False)
     annexds = Dataset(srcobj/ 'annex').create(annex=True)
@@ -250,7 +249,7 @@ def check_clone_dataset_from_just_source(url, path):
 
 @with_tempfile(mkdir=True)
 @serve_path_via_http
-def test_clone_dataset_from_just_source(src, url):
+def test_clone_dataset_from_just_source(src=None, url=None):
     ds = Dataset(src).create()
     (ds.pathobj / 'INFO.txt').write_text('content')
     ds.save()
@@ -268,7 +267,7 @@ def test_clone_dataset_from_just_source(src, url):
     })
 @serve_path_via_http
 @with_tempfile(mkdir=True)
-def test_clone_dataladri(src, topurl, path):
+def test_clone_dataladri(src=None, topurl=None, path=None):
     # make plain git repo
     ds_path = Path(src) / 'ds'
     gr = GitRepo(ds_path, create=True)
@@ -286,7 +285,7 @@ def test_clone_dataladri(src, topurl, path):
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_clone_isnot_recursive(path_src, path_nr, path_r):
+def test_clone_isnot_recursive(path_src=None, path_nr=None, path_r=None):
     src = Dataset(path_src).create()
     src.create('subm 1')
     src.create('2')
@@ -303,7 +302,7 @@ def test_clone_isnot_recursive(path_src, path_nr, path_r):
 
 @with_tempfile
 @with_tempfile
-def test_clone_into_dataset(source_path, top_path):
+def test_clone_into_dataset(source_path=None, top_path=None):
     source = Dataset(source_path).create()
     ds = create(top_path)
     assert_repo_status(ds.path)
@@ -362,7 +361,7 @@ def test_clone_into_dataset(source_path, top_path):
 
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_notclone_known_subdataset(src_path, path):
+def test_notclone_known_subdataset(src_path=None, path=None):
     src = Dataset(src_path).create()
     sub = src.create('subm 1')
     sub_id = sub.id
@@ -393,7 +392,7 @@ def test_notclone_known_subdataset(src_path, path):
 
 
 @with_tempfile(mkdir=True)
-def test_failed_clone(dspath):
+def test_failed_clone(dspath=None):
     ds = create(dspath)
     res = ds.clone("http://nonexistingreallyanything.datalad.org/bla", "sub",
                    on_failure='ignore')
@@ -406,7 +405,7 @@ def test_failed_clone(dspath):
     'ds': {'test.txt': 'some'},
     })
 @with_tempfile
-def test_clone_missing_commit(source, clone):
+def test_clone_missing_commit(source=None, clone=None):
 
     from datalad.core.distributed.clone import clone_dataset
 
@@ -499,13 +498,14 @@ def check_reckless(annex, src_path, top_path, sharedpath):
             & stat.S_IWGRP)
 
 
-def test_reckless():
-    yield check_reckless, True
-    yield check_reckless, False
+@pytest.mark.parametrize('reckless', [True, False])
+def test_reckless(reckless):
+    check_reckless(reckless)
+
 
 @with_tempfile
 @with_tempfile
-def test_install_source_relpath(src, dest):
+def test_install_source_relpath(src=None, dest=None):
     src = Path(src)
     create(src)
     src_ = src.name
@@ -515,7 +515,7 @@ def test_install_source_relpath(src, dest):
 
 @with_tempfile
 @with_tempfile
-def test_clone_isnt_a_smartass(origin_path, path):
+def test_clone_isnt_a_smartass(origin_path=None, path=None):
     origin = create(origin_path)
     cloned = clone(origin, path,
                    result_xfm='datasets', return_type='item-or-list')
@@ -532,7 +532,7 @@ def test_clone_isnt_a_smartass(origin_path, path):
 
 
 @with_tempfile(mkdir=True)
-def test_clone_report_permission_issue(tdir):
+def test_clone_report_permission_issue(tdir=None):
     pdir = Path(tdir) / 'protected'
     pdir.mkdir()
     # make it read-only
@@ -559,7 +559,7 @@ def test_clone_report_permission_issue(tdir):
 
 @skip_if_no_network
 @with_tempfile
-def test_autoenabled_remote_msg(path):
+def test_autoenabled_remote_msg(path=None):
     # Verify that no message about a remote not been enabled is displayed
     # whenever the remote we clone is the  type=git special remote, so the name
     # of the remote might not match
@@ -571,7 +571,7 @@ def test_autoenabled_remote_msg(path):
 
 @with_sameas_remote(autoenabled=True)
 @with_tempfile(mkdir=True)
-def test_clone_autoenable_msg_handles_sameas(repo, clone_path):
+def test_clone_autoenable_msg_handles_sameas(repo=None, clone_path=None):
     ds = Dataset(repo.path)
     with swallow_logs(new_level=logging.INFO) as cml:
         res = clone(ds, clone_path, result_xfm=None, return_type='list')
@@ -642,7 +642,7 @@ def test_installationpath_from_url():
 # https://github.com/datalad/datalad/issues/3958
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_expanduser(srcpath, destpath):
+def test_expanduser(srcpath=None, destpath=None):
     src = Dataset(Path(srcpath) / 'src').create()
     dest = Dataset(Path(destpath) / 'dest').create()
 
@@ -662,7 +662,7 @@ def test_expanduser(srcpath, destpath):
             message='target path already exists and not empty, refuse to '
             'clone into target path')
         # wipe out destination, and try again
-        assert_status('ok', remove(dataset=dest, check=False))
+        assert_status('ok', remove(dataset=dest, reckless='kill'))
         # now it should do it, and clone the right one
         cloneds = clone(op.join('~', 'src'), 'dest')
         eq_(cloneds.pathobj, Path(destpath) / 'dest')
@@ -674,7 +674,7 @@ def test_expanduser(srcpath, destpath):
 
 
 @with_tempfile(mkdir=True)
-def test_cfg_originorigin(path):
+def test_cfg_originorigin(path=None):
     path = Path(path)
     origin = Dataset(path / 'origin').create()
     (origin.pathobj / 'file1.txt').write_text('content')
@@ -719,7 +719,7 @@ def test_cfg_originorigin(path):
 # test fix for gh-2601/gh-3538
 @known_failure
 @with_tempfile()
-def test_relative_submodule_url(path):
+def test_relative_submodule_url(path=None):
     Dataset(op.join(path, 'origin')).create()
     ds = Dataset(op.join(path, 'ds')).create()
     with chpwd(ds.path):
@@ -738,7 +738,7 @@ def test_relative_submodule_url(path):
 
 @with_tree(tree={"subdir": {}})
 @with_tempfile(mkdir=True)
-def test_local_url_with_fetch(path, path_other):
+def test_local_url_with_fetch(path=None, path_other=None):
     path = Path(path)
     path_other = Path(path_other)
     Dataset(path / "source").create()
@@ -838,7 +838,7 @@ def _move2store(storepath, d):
 })
 @with_tempfile(mkdir=True)
 @serve_path_via_http
-def test_ria_http(lcl, storepath, url):
+def test_ria_http(lcl=None, storepath=None, url=None):
     # create a local dataset with a subdataset
     lcl = Path(lcl)
     storepath = Path(storepath)
@@ -905,7 +905,7 @@ def test_ria_http(lcl, storepath, url):
         with assert_raises(IncompleteResultsError) as cme:
             clone('ria+{}#{}@impossible'.format(url, ds.id),
                   lcl / 'clone_failed')
-        assert_in("not found in upstream", str(cme.exception))
+        assert_in("not found in upstream", str(cme.value))
 
     # lastly test if URL rewriting is in effect
     # on the surface we clone from an SSH source identified by some custom
@@ -1069,13 +1069,11 @@ def _test_ria_postclonecfg(url, dsid, clone_path, superds):
 def _postclonetest_prepare(lcl, storepath, storepath2, link):
 
     from datalad.customremotes.ria_utils import (
-        create_store,
         create_ds_in_store,
-        get_layout_locations
-    )
-    from datalad.distributed.ora_remote import (
-        LocalIO,
+        create_store,
+        get_layout_locations,
     )
+    from datalad.distributed.ora_remote import LocalIO
 
     create_tree(lcl,
                 tree={
@@ -1159,6 +1157,7 @@ def _postclonetest_prepare(lcl, storepath, storepath2, link):
     return ds.id
 
 
+# TODO?: make parametric again on _test_ria_postclonecfg
 @known_failure_osx  # https://github.com/datalad/datalad/issues/6599
 @known_failure_windows  # https://github.com/datalad/datalad/issues/5134
 @slow  # 14 sec on travis
@@ -1176,8 +1175,9 @@ def test_ria_postclonecfg():
         id = _postclonetest_prepare(lcl, store, store2)
 
         # test cloning via ria+file://
-        yield _test_ria_postclonecfg, \
+        _test_ria_postclonecfg(
               get_local_file_url(store, compatibility='git'), id
+        )
 
         # Note: HTTP disabled for now. Requires proper implementation in ORA
         #       remote. See
@@ -1188,8 +1188,9 @@ def test_ria_postclonecfg():
         #     yield _test_ria_postclonecfg, url, id
 
         # test cloning via ria+ssh://
-        yield skip_ssh(_test_ria_postclonecfg), \
+        skip_ssh(_test_ria_postclonecfg)(
             "ssh://datalad-test:{}".format(Path(store).as_posix()), id
+        )
 
 
 @known_failure_windows
@@ -1197,7 +1198,7 @@ def test_ria_postclonecfg():
 @with_tree(tree={'somefile.txt': 'some content'})
 @with_tempfile
 @with_tempfile
-def test_no_ria_postclonecfg(dspath, storepath, clonepath):
+def test_no_ria_postclonecfg(dspath=None, storepath=None, clonepath=None):
 
     dspath = Path(dspath)
     storepath = Path(storepath)
@@ -1206,12 +1207,8 @@ def test_no_ria_postclonecfg(dspath, storepath, clonepath):
     # Test that particular configuration(s) do NOT lead to a reconfiguration
     # upon clone. (See gh-5628)
 
-    from datalad.customremotes.ria_utils import (
-        create_store,
-    )
-    from datalad.distributed.ora_remote import (
-        LocalIO,
-    )
+    from datalad.customremotes.ria_utils import create_store
+    from datalad.distributed.ora_remote import LocalIO
 
     ds = Dataset(dspath).create(force=True)
     ds.save()
@@ -1253,7 +1250,7 @@ def test_no_ria_postclonecfg(dspath, storepath, clonepath):
 @with_tempfile(mkdir=True)
 @with_tempfile
 @with_tempfile
-def test_ria_postclone_noannex(dspath, storepath, clonepath):
+def test_ria_postclone_noannex(dspath=None, storepath=None, clonepath=None):
 
     # Test for gh-5186: Cloning from local FS, shouldn't lead to annex
     # initializing origin.
@@ -1263,15 +1260,11 @@ def test_ria_postclone_noannex(dspath, storepath, clonepath):
     clonepath = Path(clonepath)
 
     from datalad.customremotes.ria_utils import (
-        create_store,
         create_ds_in_store,
-        get_layout_locations
+        create_store,
+        get_layout_locations,
     )
-    from datalad.distributed.ora_remote import (
-        LocalIO,
-    )
-
-
+    from datalad.distributed.ora_remote import LocalIO
 
     # First create a dataset in a RIA store the standard way
     somefile = dspath / 'a_file.txt'
@@ -1307,7 +1300,7 @@ def test_ria_postclone_noannex(dspath, storepath, clonepath):
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
 @serve_path_via_http
-def test_inherit_src_candidates(lcl, storepath, url):
+def test_inherit_src_candidates(lcl=None, storepath=None, url=None):
     lcl = Path(lcl)
     storepath = Path(storepath)
     # dataset with a subdataset
@@ -1345,7 +1338,7 @@ def test_inherit_src_candidates(lcl, storepath, url):
 
 @skip_if_no_network
 @with_tempfile()
-def test_ria_http_storedataladorg(path):
+def test_ria_http_storedataladorg(path=None):
     # can we clone from the store w/o any dedicated config
     ds = clone('ria+http://store.datalad.org#{}'.format(datalad_store_testds_id), path)
     ok_(ds.is_installed())
@@ -1365,8 +1358,8 @@ def test_ria_http_storedataladorg(path):
 @with_tempfile
 @with_tempfile
 @with_tempfile
-def test_ephemeral(origin_path, bare_path,
-                   clone1_path, clone2_path, clone3_path):
+def test_ephemeral(origin_path=None, bare_path=None,
+                   clone1_path=None, clone2_path=None, clone3_path=None):
 
     file_test = Path('ds') / 'test.txt'
     file_testsub = Path('ds') / 'subdir' / 'testsub.txt'
@@ -1458,7 +1451,7 @@ def test_ephemeral(origin_path, bare_path,
 
 
 @with_tempfile(mkdir=True)
-def test_clone_unborn_head(path):
+def test_clone_unborn_head(path=None):
     ds_origin = Dataset(op.join(path, "a")).create()
     repo = ds_origin.repo
     managed = repo.is_managed_branch()
@@ -1491,7 +1484,7 @@ def test_clone_unborn_head(path):
     # Make the git-annex branch the most recently updated ref so that we test
     # that it is skipped.
     with set_date(abc_ts + 2):
-        ds_origin.drop("bar", check=False)
+        ds_origin.drop("bar", reckless='kill')
     ds_origin.repo.checkout(DEFAULT_BRANCH, options=["--orphan"])
 
     ds = clone(ds_origin.path, op.join(path, "b"))
@@ -1508,7 +1501,7 @@ def test_clone_unborn_head(path):
 
 
 @with_tempfile(mkdir=True)
-def test_clone_unborn_head_no_other_ref(path):
+def test_clone_unborn_head_no_other_ref(path=None):
     ds_origin = Dataset(op.join(path, "a")).create(annex=False)
     ds_origin.repo.call_git(["update-ref", "-d",
                              "refs/heads/" + DEFAULT_BRANCH])
@@ -1518,7 +1511,7 @@ def test_clone_unborn_head_no_other_ref(path):
 
 
 @with_tempfile(mkdir=True)
-def test_clone_unborn_head_sub(path):
+def test_clone_unborn_head_sub(path=None):
     ds_origin = Dataset(op.join(path, "a")).create()
     ds_origin_sub = Dataset(op.join(path, "a", "sub")).create()
     managed = ds_origin_sub.repo.is_managed_branch()
@@ -1560,7 +1553,7 @@ def test_clone_unborn_head_sub(path):
 
 @skip_if_no_network
 @with_tempfile
-def test_gin_cloning(path):
+def test_gin_cloning(path=None):
     # can we clone a public ds anoynmously from gin and retrieve content
     ds = clone('https://gin.g-node.org/datalad/datalad-ci-target', path)
     ok_(ds.is_installed())
@@ -1582,7 +1575,7 @@ def test_gin_cloning(path):
 @with_tree(tree={"special": {"f0": "0"}})
 @serve_path_via_http
 @with_tempfile(mkdir=True)
-def test_fetch_git_special_remote(url_path, url, path):
+def test_fetch_git_special_remote(url_path=None, url=None, path=None):
     url_path = Path(url_path)
     path = Path(path)
     ds_special = Dataset(url_path / "special").create(force=True)
@@ -1612,7 +1605,7 @@ def test_fetch_git_special_remote(url_path, url, path):
 @skip_if_adjusted_branch
 @skip_if_no_network
 @with_tempfile(mkdir=True)
-def test_nonuniform_adjusted_subdataset(path):
+def test_nonuniform_adjusted_subdataset(path=None):
     # https://github.com/datalad/datalad/issues/5107
     topds = Dataset(Path(path) / "top").create()
     subds_url = 'https://github.com/datalad/testrepo--basic--r1'
@@ -1624,7 +1617,7 @@ def test_nonuniform_adjusted_subdataset(path):
 
 
 @with_tempfile
-def test_clone_recorded_subds_reset(path):
+def test_clone_recorded_subds_reset(path=None):
     path = Path(path)
     ds_a = create(path / "ds_a")
     ds_a_sub = ds_a.create("sub")
@@ -1644,7 +1637,7 @@ def test_clone_recorded_subds_reset(path):
 
 
 @with_tempfile
-def test_clone_git_clone_opts(path):
+def test_clone_git_clone_opts(path=None):
     path = Path(path)
     ds_a = create(path / "ds_a", annex=False)
 
@@ -1666,7 +1659,7 @@ def test_clone_git_clone_opts(path):
 
 @with_tempfile
 @with_tempfile
-def test_clone_url_mapping(src_path, dest_path):
+def test_clone_url_mapping(src_path=None, dest_path=None):
     src = create(src_path)
     dest = Dataset(dest_path)
     # check that the impossible doesn't work
