@@ -1,5 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -8,6 +8,7 @@
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 
 import logging
+import stat
 
 from datalad.api import (
     Dataset,
@@ -31,9 +32,10 @@ from datalad.distributed.tests.ria_utils import (
     populate_dataset,
 )
 from datalad.support.exceptions import CommandError
-from datalad.tests.utils import (
+from datalad.tests.utils_pytest import (
     SkipTest,
     assert_equal,
+    assert_false,
     assert_in,
     assert_not_in,
     assert_raises,
@@ -42,8 +44,10 @@ from datalad.tests.utils import (
     assert_true,
     has_symlink_capability,
     known_failure_windows,
+    serve_path_via_http,
     skip_if_adjusted_branch,
     skip_if_no_network,
+    skip_if_root,
     skip_ssh,
     skip_wo_symlink_capability,
     slow,
@@ -56,24 +60,17 @@ from datalad.utils import Path
 # Note, that exceptions to test for are generally CommandError since we are
 # talking to the special remote via annex.
 
-@known_failure_windows  # see gh-4469
+
 @with_tempfile
 @with_tempfile
-@with_tempfile
-def _test_initremote_basic(host, ds_path, store, link):
+def _test_initremote_basic(url, io, store, ds_path, link):
 
     ds_path = Path(ds_path)
     store = Path(store)
     link = Path(link)
     ds = Dataset(ds_path).create()
     populate_dataset(ds)
-    ds.save()
 
-    if host:
-        url = "ria+ssh://{host}{path}".format(host=host,
-                                              path=store)
-    else:
-        url = "ria+{}".format(store.as_uri())
     init_opts = common_init_opts + ['url={}'.format(url)]
 
     # fails on non-existing storage location
@@ -96,7 +93,6 @@ def _test_initremote_basic(host, ds_path, store, link):
                   )
 
     # set up store:
-    io = SSHRemoteIO(host) if host else LocalIO()
     create_store(io, store, '1')
     # still fails, since ds isn't setup in the store
     assert_raises(CommandError,
@@ -159,17 +155,54 @@ def _test_initremote_basic(host, ds_path, store, link):
             # annex too old - doesn't know --sameas
             pass
         else:
-            raise 
+            raise
     # TODO: - check output of failures to verify it's failing the right way
     #       - might require to run initremote directly to get the output
 
 
-def test_initremote_basic():
+# TODO: Skipped due to gh-4436
+@known_failure_windows
+@skip_ssh
+@with_tempfile
+def test_initremote_basic_sshurl(storepath=None):
+    _test_initremote_basic(
+        'ria+ssh://datalad-test{}'.format(Path(storepath).as_posix()), \
+        SSHRemoteIO('datalad-test'), \
+        storepath,
+    )
 
-    # TODO: Skipped due to gh-4436
-    yield known_failure_windows(skip_ssh(_test_initremote_basic)), \
-          'datalad-test'
-    yield _test_initremote_basic, None
+
+# ora remote cannot handle windows file:// URLs
+@known_failure_windows
+@with_tempfile
+def test_initremote_basic_fileurl(storepath=None):
+    _test_initremote_basic(
+        "ria+{}".format(Path(storepath).as_uri()),
+        LocalIO(),
+        storepath,
+    )
+
+
+# https://github.com/datalad/datalad/issues/6160
+@known_failure_windows
+@with_tempfile(mkdir=True)
+@serve_path_via_http
+def test_initremote_basic_httpurl(storepath=None, storeurl=None):
+    _test_initremote_basic(
+        f"ria+{storeurl}",
+        LocalIO(),
+        storepath,
+    )
+
+
+@with_tempfile(mkdir=True)
+@serve_path_via_http(use_ssl=True)
+def test_initremote_basic_httpsurl(storepath=None, storeurl=None):
+    _test_initremote_basic(
+        f"ria+{storeurl}",
+        LocalIO(),
+        storepath,
+    )
 
 
 @skip_wo_symlink_capability
@@ -182,7 +215,6 @@ def _test_initremote_alias(host, ds_path, store):
     store = Path(store)
     ds = Dataset(ds_path).create()
     populate_dataset(ds)
-    ds.save()
 
     if host:
         url = "ria+ssh://{host}{path}".format(host=host,
@@ -208,9 +240,8 @@ def _test_initremote_alias(host, ds_path, store):
 def test_initremote_alias():
 
     # TODO: Skipped due to gh-4436
-    yield known_failure_windows(skip_ssh(_test_initremote_alias)), \
-          'datalad-test'
-    yield _test_initremote_alias, None
+    known_failure_windows(skip_ssh(_test_initremote_alias))('datalad-test')
+    _test_initremote_alias(None)
 
 
 
@@ -228,7 +259,6 @@ def _test_initremote_rewrite(host, ds_path, store):
     store = Path(store)
     ds = Dataset(ds_path).create()
     populate_dataset(ds)
-    ds.save()
     assert_repo_status(ds.path)
 
     url = "mystore:"
@@ -240,7 +270,7 @@ def _test_initremote_rewrite(host, ds_path, store):
     else:
         replacement = "ria+{}".format(store.as_uri())
 
-    ds.config.set("url.{}.insteadOf".format(replacement), url, where='local')
+    ds.config.set("url.{}.insteadOf".format(replacement), url, scope='local')
 
     # set up store:
     io = SSHRemoteIO(host) if host else LocalIO()
@@ -266,9 +296,8 @@ def _test_initremote_rewrite(host, ds_path, store):
 
 def test_initremote_rewrite():
     # TODO: Skipped due to gh-4436
-    yield known_failure_windows(skip_ssh(_test_initremote_rewrite)), \
-          'datalad-test'
-    yield _test_initremote_rewrite, None
+    known_failure_windows(skip_ssh(_test_initremote_rewrite))('datalad-test')
+    _test_initremote_rewrite(None)
 
 
 @known_failure_windows  # see gh-4469
@@ -282,7 +311,6 @@ def _test_remote_layout(host, dspath, store, archiv_store):
     archiv_store = Path(archiv_store)
     ds = Dataset(dspath).create()
     populate_dataset(ds)
-    ds.save()
     assert_repo_status(ds.path)
 
     # set up store:
@@ -307,14 +335,14 @@ def _test_remote_layout(host, dspath, store, archiv_store):
     ds.repo.init_remote('store', options=init_opts)
 
     # copy files into the RIA store
-    ds.repo.copy_to('.', 'store')
+    ds.push('.', to='store')
 
     # we should see the exact same annex object tree
     dsgit_dir, archive_dir, dsobj_dir = \
         get_layout_locations(1, store, ds.id)
     store_objects = get_all_files(dsobj_dir)
     local_objects = get_all_files(ds.pathobj / '.git' / 'annex' / 'objects')
-    assert_equal(len(store_objects), 2)
+    assert_equal(len(store_objects), 4)
 
     if not ds.repo.is_managed_branch():
         # with managed branches the local repo uses hashdirlower instead
@@ -345,13 +373,27 @@ def _test_remote_layout(host, dspath, store, archiv_store):
         # now fsck the new remote to get the new special remote indexed
         ds.repo.fsck(remote='archive', fast=True)
         assert_equal(len(ds.repo.whereis('one.txt')), len(whereis) + 1)
+        # test creating an archive with filters on files
+        ds.export_archive_ora(archive_dir / 'archive2.7z', annex_wanted='(include=*.txt)')
+        # test with wanted expression of a specific remote
+        ds.repo.set_preferred_content("wanted", "include=subdir/*", remote="store")
+        ds.export_archive_ora(archive_dir / 'archive3.7z', remote="store")
+        # test with the current sha
+        ds.export_archive_ora(
+            archive_dir / 'archive4.7z',
+            froms=ds.repo.get_revisions()[1],
+            )
 
 
 @slow  # 12sec + ? on travis
+# TODO: Skipped due to gh-4436
+@known_failure_windows
+@skip_ssh
+def test_remote_layout_ssh():
+    _test_remote_layout('datalad-test')
+
 def test_remote_layout():
-    # TODO: Skipped due to gh-4436
-    yield known_failure_windows(skip_ssh(_test_remote_layout)), 'datalad-test'
-    yield _test_remote_layout, None
+    _test_remote_layout(None)
 
 
 @known_failure_windows  # see gh-4469
@@ -364,7 +406,6 @@ def _test_version_check(host, dspath, store):
 
     ds = Dataset(dspath).create()
     populate_dataset(ds)
-    ds.save()
     assert_repo_status(ds.path)
 
     # set up store:
@@ -384,7 +425,7 @@ def _test_version_check(host, dspath, store):
     # add special remote
     init_opts = common_init_opts + ['url={}'.format(store_url)]
     ds.repo.init_remote('store', options=init_opts)
-    ds.repo.copy_to('.', 'store')
+    ds.push('.', to='store')
 
     # check version files
     remote_ds_tree_version_file = store / 'ria-layout-version'
@@ -431,19 +472,23 @@ def _test_version_check(host, dspath, store):
 
     # TODO: use self.annex.error in special remote and see whether we get an
     #       actual error result
-    assert_raises(CommandError,
-                  ds.repo.copy_to, 'new_file', 'store')
+    with assert_raises(CommandError):
+        ds.push('new_file', to='store')
 
     # However, we can force it by configuration
-    ds.config.add("annex.ora-remote.store.force-write", "true", where='local')
-    ds.repo.copy_to('new_file', 'store')
+    ds.config.add("annex.ora-remote.store.force-write", "true", scope='local')
+    ds.push('new_file', to='store')
 
 
 @slow  # 17sec + ? on travis
-def test_version_check():
+@skip_ssh
+@known_failure_windows
+def test_version_check_ssh():
     # TODO: Skipped due to gh-4436
-    yield known_failure_windows(skip_ssh(_test_version_check)), 'datalad-test'
-    yield _test_version_check, None
+    _test_version_check('datalad-test')
+
+def test_version_check():
+    _test_version_check(None)
 
 
 # git-annex-testremote is way too slow on crippled FS.
@@ -462,7 +507,6 @@ def _test_gitannex(host, store, dspath):
     ds = Dataset(dspath).create()
 
     populate_dataset(ds)
-    ds.save()
     assert_repo_status(ds.path)
 
     # set up store:
@@ -563,15 +607,15 @@ def _test_binary_data(host, store, dspath):
 
 def test_binary_data():
     # TODO: Skipped due to gh-4436
-    yield known_failure_windows(skip_ssh(_test_binary_data)), 'datalad-test'
-    yield skip_if_no_network(_test_binary_data), None
+    known_failure_windows(skip_ssh(_test_binary_data))('datalad-test')
+    skip_if_no_network(_test_binary_data)(None)
 
 
 @known_failure_windows
 @with_tempfile
 @with_tempfile
 @with_tempfile
-def test_push_url(storepath, dspath, blockfile):
+def test_push_url(storepath=None, dspath=None, blockfile=None):
 
     dspath = Path(dspath)
     store = Path(storepath)
@@ -580,7 +624,6 @@ def test_push_url(storepath, dspath, blockfile):
 
     ds = Dataset(dspath).create()
     populate_dataset(ds)
-    ds.save()
     assert_repo_status(ds.path)
 
     # set up store:
@@ -624,17 +667,19 @@ def test_push_url(storepath, dspath, blockfile):
     assert_in(store_uuid, known_sources)
 
 
-@skip_if_no_network
+# create-sibling-ria cannot handle windows paths
 @known_failure_windows
 @with_tempfile
 @with_tempfile
-def test_url_keys(dspath, storepath):
+@with_tempfile(mkdir=True)
+@serve_path_via_http
+def test_url_keys(dspath=None, storepath=None, httppath=None, httpurl=None):
     ds = Dataset(dspath).create()
     repo = ds.repo
     filename = 'url_no_size.html'
     # URL-type key without size
     repo.call_annex([
-        'addurl', '--relaxed', '--raw', '--file', filename, 'http://example.com',
+        'addurl', '--relaxed', '--raw', '--file', filename, httpurl,
     ])
     ds.save()
     # copy target
@@ -642,6 +687,7 @@ def test_url_keys(dspath, storepath):
         name='ria',
         url='ria+file://{}'.format(storepath),
         storage_sibling='only',
+        new_store_ok=True
     )
     ds.get(filename)
     repo.call_annex(['copy', '--to', 'ria', filename])
@@ -649,14 +695,14 @@ def test_url_keys(dspath, storepath):
     # in the store and on the web
     assert_equal(len(ds.repo.whereis(filename)), 2)
     # try download, but needs special permissions to even be attempted
-    ds.config.set('annex.security.allow-unverified-downloads', 'ACKTHPPT', where='local')
+    ds.config.set('annex.security.allow-unverified-downloads', 'ACKTHPPT', scope='local')
     repo.call_annex(['copy', '--from', 'ria', filename])
     assert_equal(len(ds.repo.whereis(filename)), 3)
     # smoke tests that execute the remaining pieces with the URL key
     repo.call_annex(['fsck', '-f', 'ria'])
     assert_equal(len(ds.repo.whereis(filename)), 3)
     # mapped key in whereis output
-    assert_in('%%example', repo.call_annex(['whereis', filename]))
+    assert_in('127.0.0.1', repo.call_annex(['whereis', filename]))
 
     repo.call_annex(['move', '-f', 'ria', filename])
     # check that it does not magically reappear, because it actually
@@ -671,3 +717,86 @@ def test_sanitize_key():
                 ('/%&:', '%&s&a&c'),
             ):
         assert_equal(_sanitize_key(i), o)
+
+
+# Skipping on adjusted branch as a proxy for crippledFS. Write permissions of
+# the owner on a directory can't be revoked on VFAT. "adjusted branch" is a
+# bit broad but covers the CI cases. And everything RIA/ORA doesn't currently
+# properly run on crippled/windows anyway. Needs to be more precise when
+# RF'ing will hopefully lead to support on windows in principle.
+@skip_if_adjusted_branch
+@known_failure_windows
+@with_tempfile
+@with_tempfile
+def _test_permission(host, storepath, dspath):
+
+    # Test whether ORA correctly revokes and obtains write permissions within
+    # the annex object tree. That is: Revoke after ORA pushed a key to store
+    # in order to allow the object tree to safely be used with an ephemeral
+    # clone. And on removal obtain write permissions, like annex would
+    # internally on a drop (but be sure to restore if something went wrong).
+
+    dspath = Path(dspath)
+    storepath = Path(storepath)
+    ds = Dataset(dspath).create()
+    populate_dataset(ds)
+    ds.save()
+    assert_repo_status(ds.path)
+    testfile = 'one.txt'
+
+    # set up store:
+    io = SSHRemoteIO(host) if host else LocalIO()
+    if host:
+        store_url = "ria+ssh://{host}{path}".format(host=host,
+                                                    path=storepath)
+    else:
+        store_url = "ria+{}".format(storepath.as_uri())
+
+    create_store(io, storepath, '1')
+    create_ds_in_store(io, storepath, ds.id, '2', '1')
+    _, _, obj_tree = get_layout_locations(1, storepath, ds.id)
+    assert_true(obj_tree.is_dir())
+    file_key_in_store = obj_tree / 'X9' / '6J' / 'MD5E-s8--7e55db001d319a94b0b713529a756623.txt' / 'MD5E-s8--7e55db001d319a94b0b713529a756623.txt'
+
+    init_opts = common_init_opts + ['url={}'.format(store_url)]
+    ds.repo.init_remote('store', options=init_opts)
+
+    store_uuid = ds.siblings(name='store',
+                             return_type='item-or-list')['annex-uuid']
+    here_uuid = ds.siblings(name='here',
+                            return_type='item-or-list')['annex-uuid']
+
+    known_sources = ds.repo.whereis(testfile)
+    assert_in(here_uuid, known_sources)
+    assert_not_in(store_uuid, known_sources)
+    assert_false(file_key_in_store.exists())
+
+    ds.repo.call_annex(['copy', testfile, '--to', 'store'])
+    known_sources = ds.repo.whereis(testfile)
+    assert_in(here_uuid, known_sources)
+    assert_in(store_uuid, known_sources)
+    assert_true(file_key_in_store.exists())
+
+    # Revoke write permissions from parent dir in-store to test whether we
+    # still can drop (if we can obtain the permissions). Note, that this has
+    # no effect on VFAT.
+    file_key_in_store.parent.chmod(file_key_in_store.parent.stat().st_mode &
+                                   ~stat.S_IWUSR)
+    # we can't directly delete; key in store should be protected
+    assert_raises(PermissionError, file_key_in_store.unlink)
+
+    # ORA can still drop, since it obtains permission to:
+    ds.repo.call_annex(['drop', testfile, '--from', 'store'])
+    known_sources = ds.repo.whereis(testfile)
+    assert_in(here_uuid, known_sources)
+    assert_not_in(store_uuid, known_sources)
+    assert_false(file_key_in_store.exists())
+
+
+@skip_ssh
+def test_obtain_permission_ssh():
+    _test_permission('datalad-test')
+
+@skip_if_root
+def test_obtain_permission_root():
+    _test_permission(None)
