@@ -1,5 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -152,6 +152,13 @@ class Install(Interface):
             metavar='SOURCE',
             doc="URL or local path of the installation source",
             constraints=EnsureStr() | EnsureNone()),
+        branch=Parameter(
+            args=("--branch",),
+            doc="""Clone source at this branch or tag. This option applies only
+            to the top-level dataset not any subdatasets that may be cloned
+            when installing recursively. Note that if the source is a RIA URL
+            with a version, it takes precedence over this option.""",
+            constraints=EnsureStr() | EnsureNone()),
         get_data=Parameter(
             args=("-g", "--get-data",),
             doc="""if given, obtain all data content too""",
@@ -168,6 +175,7 @@ class Install(Interface):
     @eval_results
     def __call__(
             path=None,
+            *,
             source=None,
             dataset=None,
             get_data=False,
@@ -175,7 +183,8 @@ class Install(Interface):
             recursive=False,
             recursion_limit=None,
             reckless=None,
-            jobs="auto"):
+            jobs="auto",
+            branch=None):
 
         # normalize path argument to be equal when called from cmdline and
         # python and nothing was passed into `path`
@@ -210,7 +219,7 @@ class Install(Interface):
                                  purpose='install')
             common_kwargs['dataset'] = dataset
         # pre-compute for results below
-        refds_path = Interface.get_refds_path(ds)
+        refds_path = ds if ds is None else ds.path
 
         # switch into the two scenarios without --source:
         # 1. list of URLs
@@ -242,6 +251,7 @@ class Install(Interface):
                         # "ignore-and-keep-going"
                         on_failure='ignore',
                         return_type='generator',
+                        result_renderer='disabled',
                         result_xfm=None,
                         result_filter=None,
                         **common_kwargs):
@@ -270,6 +280,7 @@ class Install(Interface):
                         on_failure='ignore',
                         return_type='generator',
                         result_xfm=None,
+                        result_renderer='disabled',
                         result_filter=None,
                         **common_kwargs):
                     # no post-processing of get'ed content on disk should be
@@ -355,11 +366,13 @@ class Install(Interface):
         res = Clone.__call__(
             source, path, dataset=ds, description=description,
             reckless=reckless,
+            git_clone_opts=["--branch=" + branch] if branch else None,
             # we need to disable error handling in order to have it done at
             # the very top, otherwise we are not able to order a global
             # "ignore-and-keep-going"
             result_xfm=None,
             return_type='generator',
+            result_renderer='disabled',
             result_filter=None,
             on_failure='ignore')
         # helper
@@ -390,6 +403,7 @@ class Install(Interface):
                     on_failure='ignore',
                     return_type='generator',
                     result_xfm=None,
+                    result_renderer='disabled',
                     **common_kwargs):
                 r['refds'] = refds_path
                 yield r
