@@ -1,4 +1,4 @@
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -10,31 +10,33 @@
 """
 
 from os import curdir
-from os.path import (
-    join as opj,
-    basename,
-)
+from os.path import basename
+from os.path import join as opj
 from unittest.mock import patch
 
+import pytest
+
 from datalad.api import (
+    clone,
     create,
     get,
     install,
 )
+from datalad.distribution.get import (
+    _get_flexible_source_candidates_for_submodule,
+)
 from datalad.interface.results import only_matching_paths
-from datalad.distribution.get import _get_flexible_source_candidates_for_submodule
 from datalad.support.annexrepo import AnnexRepo
 from datalad.support.exceptions import (
     InsufficientArgumentsError,
     RemoteNotAvailableError,
 )
 from datalad.support.network import get_local_file_url
-from datalad.tests.utils import (
-    ok_,
-    eq_,
-    with_tempfile,
-    with_testrepos,
-    with_tree,
+from datalad.tests.utils_testdatasets import (
+    _make_dataset_hierarchy,
+    _mk_submodule_annex,
+)
+from datalad.tests.utils_pytest import (
     create_tree,
     assert_false,
     assert_raises,
@@ -46,43 +48,32 @@ from datalad.tests.utils import (
     assert_result_count,
     assert_message,
     DEFAULT_REMOTE,
+    eq_,
+    known_failure_windows,
+    known_failure_githubci_win,
+    ok_,
     serve_path_via_http,
     skip_if_adjusted_branch,
     skip_ssh,
     skip_if_on_windows,
     slow,
-    known_failure_windows,
-    known_failure_githubci_win,
+    with_tempfile,
+    with_tree,
 )
 from datalad.utils import (
-    with_pathsep,
-    chpwd,
     Path,
+    chpwd,
     rmtree,
+    with_pathsep,
 )
+
 from ..dataset import Dataset
 
 
-def _make_dataset_hierarchy(path):
-    origin = Dataset(path).create()
-    origin_sub1 = origin.create('sub1')
-    origin_sub2 = origin_sub1.create('sub2')
-    with open(opj(origin_sub2.path, 'file_in_annex.txt'), "w") as f:
-        f.write('content2')
-    origin_sub3 = origin_sub2.create('sub3')
-    with open(opj(origin_sub3.path, 'file_in_annex.txt'), "w") as f:
-        f.write('content3')
-    origin_sub4 = origin_sub3.create('sub4')
-    origin.save(recursive=True)
-    return origin, origin_sub1, origin_sub2, origin_sub3, origin_sub4
-
-
-# AssertionError since does get extra record {'cost': 400, 'name': 'bang', 'url': 'youredead', 'from_config': True},
-@known_failure_githubci_win
 @with_tempfile
 @with_tempfile
 @with_tempfile
-def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
+def test_get_flexible_source_candidates_for_submodule(t=None, t2=None, t3=None):
     f = _get_flexible_source_candidates_for_submodule
     # for now without mocking -- let's just really build a dataset
     ds = create(t)
@@ -198,7 +189,7 @@ def test_get_flexible_source_candidates_for_submodule(t, t2, t3):
 
 @with_tempfile(mkdir=True)
 @with_tempfile(content="doesntmatter")
-def test_get_invalid_call(path, file_outside):
+def test_get_invalid_call(path=None, file_outside=None):
 
     # no argument at all:
     assert_raises(InsufficientArgumentsError, get, None)
@@ -227,9 +218,9 @@ def test_get_invalid_call(path, file_outside):
     create_tree(path, {'annexed.dat': 'some'})
     ds.save("annexed.dat")
     ds.repo.drop("annexed.dat", options=['--force'])
-    with assert_raises(RemoteNotAvailableError) as ce:
+    with assert_raises(RemoteNotAvailableError) as cme:
         ds.get("annexed.dat", source='MysteriousRemote')
-    eq_("MysteriousRemote", ce.exception.remote)
+    eq_("MysteriousRemote", cme.value.remote)
 
     res = ds.get("NotExistingFile.txt", on_failure='ignore')
     assert_status('impossible', res)
@@ -242,22 +233,31 @@ def test_get_invalid_call(path, file_outside):
         message=('path not associated with dataset %s', ds))
 
 
-@with_testrepos('basic_annex', flavors='clone')
-def test_get_single_file(path):
+@with_tempfile(mkdir=True)
+@with_tempfile(mkdir=True)
+def test_get_single_file(src=None, path=None):
+    ca = dict(result_renderer='disabled')
+    test_fname = 'test-annex.dat'
+    orig = Dataset(src).create(**ca)
+    (orig.pathobj / test_fname).write_text('some')
+    orig.save(**ca)
 
-    ds = Dataset(path)
+    ds = clone(src, path, **ca)
     ok_(ds.is_installed())
     ok_(ds.repo.file_has_content('test-annex.dat') is False)
-    result = ds.get("test-annex.dat")
+    result = ds.get("test-annex.dat", **ca)
     assert_result_count(result, 1)
     assert_status('ok', result)
     eq_(result[0]['path'], opj(ds.path, 'test-annex.dat'))
-    eq_(result[0]['annexkey'], ds.repo.get_file_key('test-annex.dat'))
-    ok_(ds.repo.file_has_content('test-annex.dat') is True)
+    annexprops = ds.repo.get_file_annexinfo('test-annex.dat',
+                                            eval_availability=True)
+    eq_(result[0]['annexkey'], annexprops['key'])
+    ok_(annexprops['has_content'])
 
 
+@pytest.mark.parametrize("override", [False, True])
 @with_tempfile(mkdir=True)
-def check_get_subdataset_inherit_reckless(override, path):
+def test_get_subdataset_inherit_reckless(path=None, *, override):
     src = Dataset(opj(path, "a")).create()
     src_subds = src.create("sub")
     src_subds.create("subsub")
@@ -280,19 +280,15 @@ def check_get_subdataset_inherit_reckless(override, path):
             None if override else "true")
 
 
-def test_get_subdataset_inherit_reckless():
-    yield check_get_subdataset_inherit_reckless, False
-    yield check_get_subdataset_inherit_reckless, True
-
-
 @with_tree(tree={'file1.txt': 'whatever 1',
                  'file2.txt': 'whatever 2',
                  'file3.txt': 'whatever 3',
                  'file4.txt': 'whatever 4'})
 @serve_path_via_http
 @with_tempfile(mkdir=True)
-def test_get_multiple_files(path, url, ds_dir):
+def test_get_multiple_files(path=None, url=None, ds_dir=None):
     from os import listdir
+
     from datalad.support.network import RI
 
     file_list = [f for f in listdir(path) if not f.startswith('.')]
@@ -337,7 +333,7 @@ def test_get_multiple_files(path, url, ds_dir):
                                 'file4.txt': 'something'
                             }}})
 @with_tempfile(mkdir=True)
-def test_get_recurse_dirs(o_path, c_path):
+def test_get_recurse_dirs(o_path=None, c_path=None):
 
     # prepare source:
     origin = Dataset(o_path).create(force=True)
@@ -373,15 +369,14 @@ def test_get_recurse_dirs(o_path, c_path):
     ok_(ds.repo.file_has_content('file1.txt') is True)
 
 
-# https://github.com/datalad/datalad/pull/3975/checks?check_run_id=369789022#step:8:541
-@known_failure_windows
 @slow  # 15.1496s
-@with_testrepos('submodule_annex', flavors='local')
 @with_tempfile(mkdir=True)
-def test_get_recurse_subdatasets(src, path):
+@with_tempfile(mkdir=True)
+def test_get_recurse_subdatasets(src=None, path=None):
+    _mk_submodule_annex(src, 'test-annex.dat', 'irrelevant')
 
-    ds = install(
-        path, source=src,
+    ds = clone(
+        src, path,
         result_xfm='datasets', return_type='item-or-list')
 
     # ask for the two subdatasets specifically. This will obtain them,
@@ -462,9 +457,10 @@ def test_get_recurse_subdatasets(src, path):
     ok_(subds2.repo.file_has_content('test-annex.dat') is False)
 
 
-@with_testrepos('submodule_annex', flavors='local')
 @with_tempfile(mkdir=True)
-def test_get_greedy_recurse_subdatasets(src, path):
+@with_tempfile(mkdir=True)
+def test_get_greedy_recurse_subdatasets(src=None, path=None):
+    _mk_submodule_annex(src, 'test-annex.dat', 'irrelevant')
 
     ds = install(
         path, source=src,
@@ -480,9 +476,10 @@ def test_get_greedy_recurse_subdatasets(src, path):
     ok_(subds2.repo.file_has_content('test-annex.dat') is True)
 
 
-@with_testrepos('submodule_annex', flavors='local')
 @with_tempfile(mkdir=True)
-def test_get_install_missing_subdataset(src, path):
+@with_tempfile(mkdir=True)
+def test_get_install_missing_subdataset(src=None, path=None):
+    _mk_submodule_annex(src, 'test-annex.dat', 'irrelevant')
 
     ds = install(
         path=path, source=src,
@@ -512,7 +509,7 @@ def test_get_install_missing_subdataset(src, path):
 #                  'subds': {'file_in_annex.txt': 'content'}})
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_get_mixed_hierarchy(src, path):
+def test_get_mixed_hierarchy(src=None, path=None):
 
     origin = Dataset(src).create(annex=False)
     origin_sub = origin.create('subds')
@@ -542,7 +539,7 @@ def test_get_mixed_hierarchy(src, path):
 @slow  # 20 sec
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_get_autoresolve_recurse_subdatasets(src, path):
+def test_get_autoresolve_recurse_subdatasets(src=None, path=None):
 
     origin = Dataset(src).create()
     origin_sub = origin.create('sub')
@@ -554,11 +551,11 @@ def test_get_autoresolve_recurse_subdatasets(src, path):
     ds = install(
         path, source=src,
         result_xfm='datasets', return_type='item-or-list')
-    eq_(len(ds.subdatasets(fulfilled=True)), 0)
+    eq_(len(ds.subdatasets(state='present')), 0)
 
     with chpwd(ds.path):
         results = get(opj(ds.path, 'sub'), recursive=True, result_xfm='datasets')
-    eq_(len(ds.subdatasets(fulfilled=True, recursive=True)), 2)
+    eq_(len(ds.subdatasets(state='present', recursive=True)), 2)
     subsub = Dataset(opj(ds.path, 'sub', 'subsub'))
     ok_(subsub.is_installed())
     assert_in(subsub, results)
@@ -570,7 +567,7 @@ def test_get_autoresolve_recurse_subdatasets(src, path):
 @slow  # 92sec
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_recurse_existing(src, path):
+def test_recurse_existing(src=None, path=None):
     origin_ds = _make_dataset_hierarchy(src)
 
     # make sure recursion_limit works as expected across a range of depths
@@ -612,7 +609,7 @@ def test_recurse_existing(src, path):
 @slow  # 33sec
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_get_in_unavailable_subdataset(src, path):
+def test_get_in_unavailable_subdataset(src=None, path=None):
     _make_dataset_hierarchy(src)
     root = install(
         path, source=src,
@@ -637,7 +634,7 @@ def test_get_in_unavailable_subdataset(src, path):
 
 @with_tempfile(mkdir=True)
 @with_tempfile(mkdir=True)
-def test_gh3356(src, path):
+def test_gh3356(src=None, path=None):
     # create toy version of gh-3356 scenario
     origin = Dataset(src).create()
     origin_sub = origin.create(origin.pathobj / 'subdir'/ 'subds')
@@ -659,7 +656,7 @@ def test_gh3356(src, path):
     assert_result_count(res, 2, action='get', type='file', status='ok')
     # status must report content for two files
     assert_result_count(
-        clone.status(recursive=True, annex='all', report_filetype='eval'), 2,
+        clone.status(recursive=True, annex='all'), 2,
         action='status', has_content=True)
 
 
@@ -669,7 +666,7 @@ def test_gh3356(src, path):
 @skip_if_on_windows
 @skip_ssh
 @with_tempfile(mkdir=True)
-def test_get_subdataset_direct_fetch(path):
+def test_get_subdataset_direct_fetch(path=None):
     path = Path(path)
     origin = Dataset(path / "origin").create()
     for sub in ["s0", "s1"]:
@@ -686,14 +683,14 @@ def test_get_subdataset_direct_fetch(path):
     # Tweak the configuration of s0 to make the direct fetch fail.
     # Disallow direct oid fetch (default).
     s0.repo.config.set("uploadpack.allowAnySHA1InWant", "false",
-                       where="local")
+                       scope="local")
     # Configure the fetcher to avoid v2, which allows fetching unadvertised
     # objects regardless of the value of uploadpack.allowAnySHA1InWant.
-    s0.repo.config.set("protocol.version", "0", where="local")
+    s0.repo.config.set("protocol.version", "0", scope="local")
 
     # Configure s1 to succeed with direct fetch.
     s1.repo.config.set("uploadpack.allowAnySHA1InWant", "true",
-                       where="local")
+                       scope="local")
 
     clone = install(
         str(path / "clone"),
@@ -707,18 +704,18 @@ def test_get_subdataset_direct_fetch(path):
 
 
 @with_tempfile()
-def test_get_relays_command_errors(path):
+def test_get_relays_command_errors(path=None):
     ds = Dataset(path).create()
     (ds.pathobj / "foo").write_text("foo")
     ds.save()
-    ds.drop("foo", check=False)
+    ds.drop("foo", reckless='kill')
     assert_result_count(
-        ds.get("foo", on_failure="ignore", result_renderer=None),
+        ds.get("foo", on_failure="ignore", result_renderer='disabled'),
         1, action="get", type="file", status="error")
 
 
 @with_tempfile()
-def test_missing_path_handling(path):
+def test_missing_path_handling(path=None):
     ds = Dataset(path).create()
     ds.save()
 
@@ -732,7 +729,6 @@ def test_missing_path_handling(path):
 
     with \
             patch("datalad.distribution.get._get_targetpaths") as get_target_path, \
-            patch("datalad.distribution.get.Interface.get_refds_path") as get_refds_path, \
             patch("datalad.distribution.get.require_dataset") as require_dataset, \
             patch("datalad.distribution.get._install_targetpath") as _install_targetpath, \
             patch("datalad.distribution.get.Subdatasets") as subdatasets:
@@ -740,7 +736,6 @@ def test_missing_path_handling(path):
         get_target_path.return_value = [{
             "status": "error"
         }]
-        get_refds_path.return_value = None
         require_dataset.return_value = refds
         _install_targetpath.return_value = [{
             "status": "notneeded",
@@ -757,6 +752,7 @@ def test_missing_path_handling(path):
         ds.get("foo")
 
 
+@slow  # started to >~30sec. https://github.com/datalad/datalad/issues/6412
 @known_failure_windows  # create-sibling-ria + ORA not fit for windows
 @with_tempfile
 @with_tempfile
@@ -764,8 +760,8 @@ def test_missing_path_handling(path):
                  'sub2': {'file2.txt': 'content 2'}})
 @with_tempfile
 @with_tempfile
-def test_source_candidate_subdataset(store1, store2, intermediate,
-                                     super, clone):
+def test_source_candidate_subdataset(store1=None, store2=None, intermediate=None,
+                                     super=None, clone=None):
 
     # This tests the scenario of gh-6159.
     # However, the actual point is to test that `get` does not overwrite a
@@ -778,12 +774,13 @@ def test_source_candidate_subdataset(store1, store2, intermediate,
     ds.create("sub2", force=True)
     ds.save(recursive=True)
     ria_url_1 = "ria+" + get_local_file_url(store1, compatibility='git')
-    ds.create_sibling_ria(ria_url_1, "firststore", recursive=True)
+    ds.create_sibling_ria(ria_url_1, "firststore", recursive=True,
+                          new_store_ok=True)
     ds.push(".", to="firststore", recursive=True)
     superds = Dataset(super).create()
     superds.clone(source=ria_url_1 + "#" + ds.id, path="intermediate")
     ria_url_2 = "ria+" + get_local_file_url(store2, compatibility='git')
-    superds.create_sibling_ria(ria_url_2, "secondstore")
+    superds.create_sibling_ria(ria_url_2, "secondstore", new_store_ok=True)
     superds.push(".", to="secondstore")
 
     cloneds = install(clone, source=ria_url_2 + "#" + superds.id)
