@@ -1,5 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -9,53 +9,57 @@
 """Create and update a dataset from a list of URLs.
 """
 
-from collections import defaultdict
-from collections.abc import Mapping
-
 import json
 import logging
 import os
 import re
 import string
 import sys
-
+from collections import defaultdict
+from collections.abc import Mapping
 from functools import partial
 from urllib.parse import urlparse
 
-from datalad.support.external_versions import external_versions
 import datalad.support.path as op
 from datalad.distribution.dataset import resolve_path
-from datalad.dochelpers import (
-    single_or_plural,
+from datalad.dochelpers import single_or_plural
+from datalad.interface.base import (
+    Interface,
+    build_doc,
 )
-from datalad.log import log_progress, with_result_progress
-from datalad.interface.base import Interface
-from datalad.interface.base import build_doc
-from datalad.interface.utils import (
-    default_result_renderer,
-    render_action_summary,
-)
-from datalad.interface.results import annexjson2result, get_status_dict
 from datalad.interface.common_opts import (
     jobs_opt,
     nosave_opt,
 )
+from datalad.interface.results import (
+    annexjson2result,
+    get_status_dict,
+)
+from datalad.interface.utils import (
+    generic_result_renderer,
+    render_action_summary,
+)
+from datalad.log import (
+    log_progress,
+    with_result_progress,
+)
 from datalad.support.exceptions import (
     CapturedException,
     CommandError,
 )
+from datalad.support.external_versions import external_versions
 from datalad.support.itertools import groupby_sorted
 from datalad.support.network import get_url_filename
-from datalad.support.path import split_ext
 from datalad.support.parallel import (
     ProducerConsumerProgressLog,
     no_parentds_in_futures,
 )
+from datalad.support.path import split_ext
 from datalad.support.s3 import get_versioned_url
 from datalad.utils import (
+    Path,
     ensure_list,
     get_suggestions_msg,
-    Path,
     unlink,
 )
 
@@ -178,7 +182,7 @@ def clean_meta_args(args):
 def get_subpaths(filename):
     """Convert "//" marker in `filename` to a list of subpaths.
 
-    >>> from datalad.plugin.addurls import get_subpaths
+    >>> from datalad.local.addurls import get_subpaths
     >>> get_subpaths("p1/p2//p3/p4//file")
     ('p1/p2/p3/p4/file', ['p1/p2', 'p1/p2/p3/p4'])
 
@@ -875,9 +879,9 @@ class RegisterUrl(object):
         self.repo = repo or ds.repo
         self._err_res = get_status_dict(action="addurls", ds=self.ds,
                                         type="file", status="error")
-        self.use_pointer = repo.is_managed_branch()
+        self.use_pointer = self.repo.is_managed_branch()
         self._avoid_fromkey = self.use_pointer and \
-            not repo._check_version_kludges("fromkey-supports-unlocked")
+            not self.repo._check_version_kludges("fromkey-supports-unlocked")
 
     def examinekey(self, parsed_key, filename, migrate=False):
         opts = []
@@ -961,13 +965,21 @@ class BatchedRegisterUrl(RegisterUrl):
         super().__init__(ds, repo)
         self._batch_commands = {}
 
-    def _batch(self, command, batch_input,
-               output_proc=None, json=False, batch_options=None):
+    def _batch(self,
+               command,
+               batch_input,
+               output_proc=None,
+               json=False,
+               batch_options=None):
+
         bcmd = self._batch_commands.get(command)
         if not bcmd:
             repo = self.repo
             bcmd = repo._batched.get(
-                command, path=repo.path, json=json, output_proc=output_proc,
+                codename=command,
+                path=repo.path,
+                json=json,
+                output_proc=output_proc,
                 annex_options=batch_options)
             self._batch_commands[command] = bcmd
         return bcmd(batch_input)
@@ -1129,7 +1141,7 @@ class Addurls(Interface):
     To download each link into a file name composed of the 'who' and 'ext'
     fields, we could run::
 
-      $ datalad addurls -d avatar_ds --fast avatars.csv '{link}' '{who}.{ext}'
+      $ datalad addurls -d avatar_ds avatars.csv '{link}' '{who}.{ext}'
 
     The `-d avatar_ds` is used to create a new dataset in "$PWD/avatar_ds".
 
@@ -1137,7 +1149,7 @@ class Addurls(Interface):
     "avatars" subdirectory, we could use "//" in the `FILENAME-FORMAT`
     argument::
 
-      $ datalad addurls --fast avatars.csv '{link}' 'avatars//{who}.{ext}'
+      $ datalad addurls avatars.csv '{link}' 'avatars//{who}.{ext}'
 
     If the information is represented as JSON lines instead of comma separated
     values or a JSON array, you can use a utility like jq to transform the JSON
@@ -1153,10 +1165,16 @@ class Addurls(Interface):
        --batch --with-files'.
     """
 
-    from datalad.distribution.dataset import datasetmethod
+    from datalad.distribution.dataset import (
+        EnsureDataset,
+        datasetmethod,
+    )
     from datalad.interface.utils import eval_results
-    from datalad.distribution.dataset import EnsureDataset
-    from datalad.support.constraints import EnsureChoice, EnsureNone, EnsureStr
+    from datalad.support.constraints import (
+        EnsureChoice,
+        EnsureNone,
+        EnsureStr,
+    )
     from datalad.support.param import Parameter
 
     _params_ = dict(
@@ -1254,7 +1272,13 @@ class Addurls(Interface):
             args=("--fast",),
             action="store_true",
             doc="""If True, add the URLs, but don't download their content.
-            Underneath, this passes the --fast flag to `git annex addurl`."""),
+            WARNING: ONLY USE THIS OPTION IF YOU UNDERSTAND THE CONSEQUENCES.
+            If the content of the URLs is not downloaded, then datalad
+            will refuse to retrieve the contents with `datalad get <file>` by default
+            because the content of the URLs is not verified.  Add 
+            `annex.security.allow-unverified-downloads = ACKTHPPT` to your git config to bypass
+            the safety check.  Underneath, this passes the
+            `--fast` flag to `git annex addurl`."""),
         ifexists=Parameter(
             args=("--ifexists",),
             doc="""What to do if a constructed file name already exists.  The
@@ -1306,7 +1330,9 @@ class Addurls(Interface):
     @staticmethod
     @datasetmethod(name='addurls')
     @eval_results
-    def __call__(dataset, urlfile, urlformat, filenameformat,
+    def __call__(urlfile, urlformat, filenameformat,
+                 *,
+                 dataset=None,
                  input_type="ext", exclude_autometa=None, meta=None, key=None,
                  message=None, dry_run=False, fast=False, ifexists=None,
                  missing_value=None, save=True, version_urls=False,
@@ -1320,7 +1346,10 @@ class Addurls(Interface):
 
         from requests.exceptions import RequestException
 
-        from datalad.distribution.dataset import Dataset, require_dataset
+        from datalad.distribution.dataset import (
+            Dataset,
+            require_dataset,
+        )
         from datalad.support.annexrepo import AnnexRepo
 
         lgr = logging.getLogger("datalad.local.addurls")
@@ -1414,6 +1443,7 @@ class Addurls(Interface):
             yield from ds.create(
                 result_xfm=None,
                 return_type='generator',
+                result_renderer='disabled',
                 cfg_proc=cfg_proc)
 
         annex_options = ["--fast"] if fast else []
@@ -1451,6 +1481,7 @@ class Addurls(Interface):
             else:
                 for res in subds.create(result_xfm=None,
                                         cfg_proc=cfg_proc,
+                                        result_renderer='disabled',
                                         return_type='generator'):
                     if res.get("action") == "create":
                         res["addurls.refds"] = ds_path
@@ -1565,6 +1596,7 @@ filename_format='{filenameformat}'"""
                 list(files_to_add),
                 message=message_addurls,
                 jobs=jobs,
+                result_renderer='disabled',
                 return_type='generator')
 
     @staticmethod
@@ -1572,7 +1604,7 @@ filename_format='{filenameformat}'"""
         refds = res.get("addurls.refds")
         if refds:
             res = dict(res, refds=refds)
-        default_result_renderer(res)
+        generic_result_renderer(res)
 
     custom_result_summary_renderer_pass_summary = True
 
