@@ -13,6 +13,7 @@
 __docformat__ = 'restructuredtext'
 
 import os
+import tempfile
 from glob import glob
 from os import unlink
 from os.path import (
@@ -39,7 +40,8 @@ from datalad.support.exceptions import (
     CommandError,
     NoDatasetFound,
 )
-from datalad.tests.utils import (
+from datalad.support.external_versions import external_versions
+from datalad.tests.utils_pytest import (
     assert_cwd_unchanged,
     assert_equal,
     assert_false,
@@ -59,6 +61,7 @@ from datalad.tests.utils import (
     ok_file_has_content,
     ok_file_under_git,
     serve_path_via_http,
+    skip_if,
     skip_if_adjusted_branch,
     swallow_outputs,
     with_tempfile,
@@ -67,6 +70,7 @@ from datalad.tests.utils import (
 from datalad.utils import (
     chpwd,
     find_files,
+    get_tempfile_kwargs,
     getpwd,
     on_windows,
     rmtemp,
@@ -99,7 +103,7 @@ treeargs = dict(
 @with_tree(**treeargs)
 @serve_path_via_http()
 @with_tempfile(mkdir=True)
-def test_add_archive_dirs(path_orig, url, repo_path):
+def test_add_archive_dirs(path_orig=None, url=None, repo_path=None):
     # change to repo_path
     with chpwd(repo_path):
         # create annex repo
@@ -107,8 +111,7 @@ def test_add_archive_dirs(path_orig, url, repo_path):
         repo = ds.repo
         # add archive to the repo so we could test
         with swallow_outputs():
-            repo.add_urls([opj(url, '1.tar.gz')],
-                             options=["--pathdepth", "-1"])
+            repo.add_url_to_file('1.tar.gz', opj(url, '1.tar.gz'))
         repo.commit("added 1.tar.gz")
 
         # test with excludes and annex options
@@ -199,7 +202,7 @@ tree4uargs = dict(
 @with_tree(**tree1args)
 @serve_path_via_http()
 @with_tempfile(mkdir=True)
-def test_add_archive_content(path_orig, url, repo_path):
+def test_add_archive_content(path_orig=None, url=None, repo_path=None):
     with chpwd(repo_path):
         # TODO we need to be able to pass path into add_archive_content
         # We could mock but I mean for the API
@@ -223,13 +226,12 @@ def test_add_archive_content(path_orig, url, repo_path):
             message="Can not add archive outside of the dataset"
         )
 
-
         # Let's add first archive to the repo so we could test
         with swallow_outputs():
-            repo.add_urls([opj(url, '1.tar.gz')], options=["--pathdepth", "-1"])
+            repo.add_url_to_file('1.tar.gz', opj(url, '1.tar.gz'))
             for s in range(1, 5):
-                repo.add_urls([opj(url, '%du/1.tar.gz' % s)],
-                                 options=["--pathdepth", "-2"])
+                repo.add_url_to_file('%du/1.tar.gz' % s,
+                                     opj(url, '%du/1.tar.gz' % s))
             repo.commit("added 1.tar.gz")
 
         key_1tar = repo.get_file_annexinfo('1.tar.gz')['key']  # will be used in the test later
@@ -317,8 +319,7 @@ def test_add_archive_content(path_orig, url, repo_path):
         # Let's add first archive to the repo so we could test
         # named the same way but different content
         with swallow_outputs():
-            repo.add_urls([opj(url, 'd1', '1.tar.gz')], options=["--pathdepth", "-1"],
-                          cwd=getpwd())  # invoke under current subdir
+            repo.add_url_to_file('d1/1.tar.gz', opj(url, 'd1', '1.tar.gz'))
         repo.commit("added 1.tar.gz in d1")
 
         def d2_basic_checks():
@@ -372,14 +373,13 @@ def test_add_archive_content(path_orig, url, repo_path):
 @with_tree(**tree1args)
 @serve_path_via_http()
 @with_tempfile(mkdir=True)
-def test_add_archive_content_strip_leading(path_orig, url, repo_path):
+def test_add_archive_content_strip_leading(path_orig=None, url=None, repo_path=None):
     with chpwd(repo_path):
         ds = Dataset(repo_path).create(force=True)
         repo = ds.repo
         # Let's add first archive to the repo so we could test
         with swallow_outputs():
-            repo.add_urls([opj(url, '1.tar.gz')],
-                             options=["--pathdepth", "-1"])
+            repo.add_url_to_file('1.tar.gz', opj(url, '1.tar.gz'))
         repo.commit("added 1.tar.gz")
 
         add_archive_content('1.tar.gz', strip_leading_dirs=True)
@@ -391,7 +391,7 @@ def test_add_archive_content_strip_leading(path_orig, url, repo_path):
 
 @assert_cwd_unchanged(ok_to_chdir=True)
 @with_tree(tree={"1.zip": {"dir": {"bar": "blah"}, "foo": "blahhhhh"}})
-def test_add_archive_content_zip(repo_path):
+def test_add_archive_content_zip(repo_path=None):
     ds = Dataset(repo_path).create(force=True)
     with chpwd(repo_path):
         with swallow_outputs():
@@ -404,7 +404,7 @@ def test_add_archive_content_zip(repo_path):
 
 @with_tree(tree={"ds": {"1.tar.gz": {"foo": "abc"}},
                  "notds": {"2.tar.gz": {"bar": "def"}}})
-def test_add_archive_content_absolute_path(path):
+def test_add_archive_content_absolute_path(path=None):
     ds = Dataset(opj(path, "ds")).create(force=True)
     repo = ds.repo
     ds.save("1.tar.gz", message="1.tar.gz")
@@ -428,7 +428,7 @@ def test_add_archive_content_absolute_path(path):
 
 @assert_cwd_unchanged(ok_to_chdir=True)
 @with_tree(**tree4uargs)
-def test_add_archive_use_archive_dir(repo_path):
+def test_add_archive_use_archive_dir(repo_path=None):
     ds = Dataset(repo_path).create(force=True)
     with chpwd(repo_path):
         # Let's add first archive to the repo with default setting
@@ -475,7 +475,7 @@ def test_add_archive_use_archive_dir(repo_path):
         },
     }
 )
-def test_add_archive_single_file(repo_path):
+def test_add_archive_single_file(repo_path=None):
     ds = Dataset(repo_path).create(force=True)
     with chpwd(repo_path):
         archives = glob('archives/*')
@@ -492,10 +492,12 @@ class TestAddArchiveOptions():
     # few tests bundled with a common setup/teardown to minimize boiler plate
     # nothing here works on windows, no even teardown(), prevent failure at the
     # origin
-    @with_tree(tree={'1.tar': {'file.txt': 'load',
-                               '1.dat': 'load2'}},
-               delete=False)
-    def setup(self, repo_path):
+    def setup(self):
+        repo_path = tempfile.mkdtemp(**get_tempfile_kwargs(prefix="tree"))
+        create_tree(
+            repo_path,
+            {'1.tar': {'file.txt': 'load',
+                       '1.dat': 'load2'}})
         self.ds = ds = Dataset(repo_path)
         ds.create(force=True)
         self.annex = ds.repo
@@ -513,6 +515,12 @@ class TestAddArchiveOptions():
                                     delete=True)
         assert_false(lexists(self.ds.pathobj / '1.tar'))
 
+    # git-annex regression
+    # https://git-annex.branchable.com/bugs/regression__58___annex_add_of_moved_file_errors_out/
+    @skip_if(
+        '10.20220624' <= external_versions['cmd:annex'] < '10.20220706',  # approx when was fixed
+        msg="buggy git-annex release"
+    )
     def test_add_archive_leading_dir(self):
         import os
         os.mkdir(self.ds.pathobj / 'sub')
@@ -570,7 +578,7 @@ class TestAddArchiveOptions():
         # there should be no .datalad temporary files hanging around
         self.assert_no_trash_left_behind()
 
-    def test_add_delete_after_and_drop_subdir(self):
+    def test_add_delete_after_and_drop_subdir(self=None):
         os.mkdir(opj(self.annex.path, 'subdir'))
         mv_out = self.annex.call_git(
             ['mv', '1.tar', 'subdir']
