@@ -6,6 +6,7 @@
 #   copyright and license terms.
 #
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
+"""Logging setup and utilities, including progress reporting"""
 
 from functools import partial
 import inspect
@@ -15,6 +16,7 @@ import sys
 import platform
 import random
 import logging.handlers
+import warnings
 
 from os.path import basename, dirname
 
@@ -23,16 +25,33 @@ from collections import defaultdict
 from .utils import is_interactive, optional_args
 from .support import ansi_colors as colors
 
-__all__ = ['ColorFormatter']
+__all__ = [
+    'ColorFormatter',
+    'LoggerHelper',
+    'filter_noninteractive_progress',
+    'log_progress',
+    'with_progress',
+    'with_result_progress',
+]
 
 # Snippets from traceback borrowed from duecredit which was borrowed from
 # PyMVPA upstream/2.4.0-39-g69ad545  MIT license (the same copyright as DataLad)
 
 
 def mbasename(s):
-    """Custom function to include directory name if filename is too common
+    """Returns an expanded basename, if the filename is deemed not informative
 
-    Also strip .py at the end
+    A '.py' extension is stripped from file name, and the containing directory
+    is prepended for too generic file names  like 'base', '__init__', and 'utils'
+
+    Parameters
+    ----------
+    s: str
+      Platform-native path
+
+    Returns
+    -------
+    str
     """
     base = basename(s)
     if base.endswith('.py'):
@@ -43,17 +62,16 @@ def mbasename(s):
 
 
 class TraceBack(object):
-    """Customized traceback to be included in debug messages
+    """Customizable traceback for inclusion debug log messages
     """
 
     def __init__(self, limit=100, collide=False):
-        """Initialize TraceBack metric
-
+        """
         Parameters
         ----------
         collide : bool
-          if True then prefix common with previous invocation gets
-          replaced with ...
+          if True, deduplicate a subsequent message by replacing a common
+          prefix string with an ellipsis.
         """
         self.__prev = ""
         self.limit = limit
@@ -66,17 +84,50 @@ class TraceBack(object):
         else:
             self.__prefix_re = None
 
+    def _extract_stack(self, limit=None):
+        """Call traceback.extract_stack() with limit parameter
+
+        Parameters
+        ----------
+        limit: int, optional
+          Limit stack trace entries (starting from the invocation point)
+          if limit is positive, or to the last `abs(limit)` entries.
+          If limit is omitted or None, all entries are printed.
+
+        Returns
+        -------
+        traceback.StackSummary
+        """
         import traceback
-        self._extract_stack = traceback.extract_stack
+        return traceback.extract_stack(limit=limit)
 
     def __call__(self):
-        ftb = self._extract_stack(limit=self.limit+10)[:-2]
-        entries = [[mbasename(x[0]), str(x[1])]
-                   for x in ftb if mbasename(x[0]) != 'logging.__init__']
-        entries = [e for e in entries if e[0] != 'unittest']
+        # get the stack description. All but the last three items, which
+        # represent the entry into this utility rather than the traceback
+        # relevant for the caller
+        ftb = self._extract_stack(limit=self.limit + 10)[:-3]
+        # each item in `ftb` is
+        # (filename, line, object, code-on-line)
+        # so entries is a list of (filename, line)
+        entries = [
+            [mbasename(x[0]), str(x[1])] for x in ftb
+            # the last entry in the filtered list will always come from
+            # this datalad/log.py facility, where the log record
+            # was emitted, it is not meaningful to always show a constant
+            # trailing end of the traceback
+            # more generally, we are hardly ever interested in traceback
+            # pieces from this file
+            if x[0] != __file__
+        ]
+        # remove more "uninformative" levels of the stack, given the space
+        # constraints of a log message
+        entries = [
+            e for e in entries
+            if e[0] not in ('unittest', 'logging.__init__')
+        ]
 
         if len(entries) > self.limit:
-            sftb = '...>'
+            sftb = '…>'
             entries = entries[-self.limit:]
         else:
             sftb = ''
@@ -84,14 +135,17 @@ class TraceBack(object):
         if not entries:
             return ""
 
-        # lets make it more consize
+        # let's make it more concise
         entries_out = [entries[0]]
         for entry in entries[1:]:
+            # if the current filename is the same as the last one on the stack
+            # only append the line number to save space
             if entry[0] == entries_out[-1][0]:
                 entries_out[-1][1] += ',%s' % entry[1]
             else:
                 entries_out.append(entry)
 
+        # format the traceback in a compact form
         sftb += '>'.join(
             ['%s:%s' % (mbasename(x[0]), x[1]) for x in entries_out]
         )
@@ -103,7 +157,7 @@ class TraceBack(object):
             common_prefix2 = self.__prefix_re.sub('', common_prefix)
 
             if common_prefix2 != "":
-                sftb = '...' + sftb[len(common_prefix2):]
+                sftb = '…' + sftb[len(common_prefix2):]
             self.__prev = prev_next
 
         return sftb
@@ -155,7 +209,13 @@ class ColorFormatter(logging.Formatter):
                                 self.use_color)
         log_env = os.environ.get('DATALAD_LOG_TRACEBACK', '')
         collide = log_env == 'collide'
-        limit = 100 if collide else int(log_env) if log_env.isdigit() else 100
+        # if an integer level is given, we add one level to make the behavior
+        # more natural. The last frame will always be where the log record
+        # was emitted by out handler. However, user is more interested in the
+        # location where the log message originates.
+        # Saying DATALAD_LOG_TRACEBACK=1 will give that in most cases, with
+        # this internal increment
+        limit = 100 if collide else int(log_env) + 1 if log_env.isdigit() else 100
         self._tb = TraceBack(collide=collide, limit=limit) if log_env else None
 
         self._mem = MemoryInfo() if os.environ.get('DATALAD_LOG_VMEM', '') else None
@@ -269,28 +329,60 @@ class ProgressHandler(logging.Handler):
             pb.clear()
 
 
-class NoProgressLog(logging.Filter):
-    def filter(self, record):
-        return not hasattr(record, 'dlm_progress')
-
-
-class OnlyProgressLog(logging.Filter):
-    def filter(self, record):
-        return hasattr(record, 'dlm_progress')
-
-
 def filter_noninteractive_progress(logger, record):
+    """Companion of log_progress() to suppress undesired progress logging
+
+    This filter is to be used with a log handler's addFilter() method
+    for the case of a non-interactive session (e.g., pipe to log file).
+
+    It inspects the log record for `dlm_progress_noninteractive_level`
+    keys that can be injected via log_progress(noninteractive_level=).
+
+    If a log-level was declared in this fashion, it will be evaluated
+    against the logger's effective level, and records are discarded
+    if their level is too low. If no log-level was declared, a log record
+    passes this filter unconditionally.
+
+    Parameters
+    ----------
+    logger: logging.Logger
+      The logger instance whose effective level to check against.
+    record:
+      The log record to inspect.
+
+    Returns
+    -------
+    bool
+    """
     level = getattr(record, "dlm_progress_noninteractive_level", None)
     return level is None or level >= logger.level
 
 
 def log_progress(lgrcall, pid, *args, **kwargs):
-    """Helper to emit a log message on the progress of some process
+    """Emit progress log messages
+
+    This helper can be used to handle progress reporting without having
+    to maintain display mode specific code.
+
+    Typical progress reporting via this function involves three types of
+    calls:
+
+    1. Start reporting progress about a process
+    2. Update progress information about a process
+    3. Report completion of a process
+
+    In order to be able to associate all three steps with a particular process,
+    the `pid` identifier is used. This is an arbitrary string that must be
+    chosen to be unique across all different, but simultaneously running
+    progress reporting activities within a Python session. For many practical
+    purposes this can be achieved by, for example, including path information
+    in the identifier.
+
+    To initialize a progress report this function is called without an
+    `update` parameter. To report a progress update, this function is called
+    with an `update` parameter. To finish a reporting on a particular activity
+    a final call without an `update` parameter is required.
 
-    Note: Whereas this helper reports on interim progress and is to be used
-    programmatically, :class:`~datalad.ui.progressbars.LogProgressBar` replaces
-    a progress bar with a single log message upon completion and can be chosen
-    by the user (config 'datalad.ui.progressbar' set to 'log').
 
     Parameters
     ----------
@@ -309,22 +401,28 @@ def log_progress(lgrcall, pid, *args, **kwargs):
       Progress report unit. Should be very brief, goes after the progress bar
       on the same line.
     update : int
-      To which quantity to advance the progress.
+      To (or by) which quantity to advance the progress. Also see `increment`.
     increment : bool
       If set, `update` is interpreted as an incremental value, not absolute.
     initial : int
       If set, start value for progress bar
     noninteractive_level : int, optional
-      When a level is specified here and progress is being logged
-      non-interactively (i.e. without progress bars), do not log the message if
-      logging is not enabled for the specified level. This is useful when you
-      want all calls to be "logged" via the progress bar, but non-interactively
-      showing a message at the `lgrcall` level for each step would be too much
-      noise. Note that the level here only determines if the record will be
-      dropped; it will still be logged at the level of `lgrcall`.
+      In a non-interactive session where progress bars are not displayed,
+      only log a progress report, if a logger's effective level includes the
+      specified level. This can be useful logging all progress is inappropriate
+      or too noisy for a log.
     maint : {'clear', 'refresh'}
+      This is a special attribute that can be used by callers that are not
+      actually reporting progress, but need to ensure that their (console)
+      output does not interfere with any possibly ongoing progress reporting.
+      Setting this attribute to 'clear' will cause the central ProgressHandler
+      to temporarily stop the display of any active progress bars. With
+      'refresh', all active progress bars will be redisplayed. After a 'clear'
+      individual progress bars would be reactivated upon the next update log
+      message, even without an explicit 'refresh'.
     """
     d = dict(
+        # inject progress-related result properties as extra data
         {'dlm_progress_{}'.format(n): v for n, v in kwargs.items()
          # initial progress might be zero, but not sending it further
          # would signal to destroy the progress bar, hence test for 'not None'
@@ -503,10 +601,8 @@ class LoggerHelper(object):
 
         Parameters
         ----------
-        target: string, optional
-          Which log target to request logger for
-        logtarget: { 'stdout', 'stderr', str }, optional
-          Where to direct the logs.  stdout and stderr stand for standard streams.
+        logtarget: {'stderr', str }, optional
+          Where to direct the logs. 'stderr' stands for the standard stream.
           Any other string is considered a filename.  Multiple entries could be
           specified comma-separated
 
@@ -514,8 +610,8 @@ class LoggerHelper(object):
         -------
         logging.Logger
         """
-        # By default mimic previously talkative behavior
-        logtarget = self._get_config('target', logtarget or 'stderr')
+        if not logtarget:
+            logtarget = self._get_config('target', 'stderr')
 
         # Allow for multiple handlers being specified, comma-separated
         if ',' in logtarget:
@@ -523,9 +619,12 @@ class LoggerHelper(object):
                 self.get_initialized_logger(logtarget=handler_)
             return self.lgr
 
-        if logtarget.lower() in ('stdout', 'stderr'):
-            loghandler = logging.StreamHandler(getattr(sys, logtarget.lower()))
+        if logtarget.lower() == 'stderr':
+            loghandler = logging.StreamHandler(sys.stderr)
             use_color = is_interactive()  # explicitly decide here
+        elif logtarget.lower() == 'stdout':
+            warnings.warn("'stdout' was discontinued as valid log target and "
+                          "will be ignored.", DeprecationWarning)
         else:
             # must be a simple filename
             # Use RotatingFileHandler for possible future parametrization to keep
