@@ -8,28 +8,27 @@
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 """Test audio extractor"""
 
-from datalad.tests.utils import (
+from datalad.tests.utils_pytest import (
+    SkipTest,
     assert_in,
     assert_not_in,
     assert_repo_status,
     assert_result_count,
     assert_status,
     eq_,
-    SkipTest,
     with_tempfile,
 )
+
 try:
     import mutagen
 except ImportError:
     raise SkipTest
 
+from os.path import dirname
+from os.path import join as opj
 from shutil import copy
-from os.path import (
-    dirname,
-    join as opj,
-)
-from datalad.api import Dataset
 
+from datalad.api import Dataset
 
 target = {
     "format": "mime:audio/mp3",
@@ -46,7 +45,7 @@ target = {
 
 
 @with_tempfile(mkdir=True)
-def test_audio(path):
+def test_audio(path=None):
     ds = Dataset(path).create()
     ds.config.add('datalad.metadata.nativetype', 'audio', scope='branch')
     copy(
