@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 """Procedure to configure additional metadata types
 
 Additional arguments: <metadata type label> [...]
@@ -27,10 +28,11 @@ for nt in sys.argv[2:]:
     ds.config.add(
         'datalad.metadata.nativetype',
         nt,
-        where='dataset',
+        scope='branch',
         reload=False)
 
 ds.save(
     path=op.join(ds.path, DATASET_CONFIG_FILE),
     message="Configure metadata type(s)",
+    result_renderer='disabled'
 )
