@@ -1,5 +1,5 @@
 # emacs: -*- mode: python-mode; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil; coding: utf-8 -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -19,9 +19,9 @@ from time import (
     time,
 )
 
-from nose.tools import timed
+import pytest
 
-from datalad.tests.utils import (
+from datalad.tests.utils_pytest import (
     OBSCURE_FILENAME,
     SkipTest,
     assert_cwd_unchanged,
@@ -43,25 +43,18 @@ from datalad.utils import (
 
 from .. import (
     CommandError,
+    KillOutput,
     Protocol,
     Runner,
     StdOutCapture,
     StdOutErrCapture,
 )
-
-
-def py2cmd(code):
-    """Helper to invoke some Python code through a cmdline invocation of
-    the Python interpreter.
-
-    This should be more portable in some cases.
-    """
-    return [sys.executable, '-c', code]
+from .utils import py2cmd
 
 
 @assert_cwd_unchanged
 @with_tempfile
-def test_runner(tempfile):
+def test_runner(tempfile=None):
     runner = Runner()
     content = 'Testing real run' if on_windows else 'Testing äöü東 real run' 
     cmd = 'echo %s > %s' % (content, tempfile)
@@ -96,17 +89,17 @@ def test_runner_stdout_capture():
 
 
 @with_tempfile(mkdir=True)
-def test_runner_failure(dir_):
+def test_runner_failure(dir_=None):
     runner = Runner()
     with assert_raises(CommandError) as cme:
         runner.run(
             py2cmd('import sys; sys.exit(53)')
         )
-    eq_(53, cme.exception.code)
+    eq_(53, cme.value.code)
 
 
 @with_tempfile(mkdir=True)
-def test_runner_fix_PWD(path):
+def test_runner_fix_PWD(path=None):
     env = os.environ.copy()
     env['PWD'] = orig_cwd = os.getcwd()
     runner = Runner(cwd=path, env=env)
@@ -119,7 +112,7 @@ def test_runner_fix_PWD(path):
 
 
 @with_tempfile(mkdir=True)
-def test_runner_cwd_encoding(path):
+def test_runner_cwd_encoding(path=None):
     env = os.environ.copy()
     # Add PWD to env so that runner will temporarily adjust it to point to cwd.
     env['PWD'] = os.getcwd()
@@ -133,7 +126,7 @@ def test_runner_cwd_encoding(path):
 
 
 @with_tempfile(mkdir=True)
-def test_runner_stdin(path):
+def test_runner_stdin(path=None):
     runner = Runner()
     fakestdin = Path(path) / 'io'
     # go for difficult content
@@ -155,18 +148,18 @@ def test_runner_stdin(path):
     assert_in(OBSCURE_FILENAME, res['stdout'])
 
 
-@timed(3)
+@pytest.mark.fail_slow(3)
 def test_runner_stdin_no_capture():
     # Ensure that stdin writing alone progresses
     runner = Runner()
     runner.run(
         py2cmd('import sys; print(sys.stdin.read()[-10:])'),
-        stdin=('ABCDEFGHIJKLMNOPQRSTUVWXYZ-' * 10000).encode('utf-8'),
+        stdin=('ABCDEFGHIJKLMNOPQRSTUVWXYZ-' * 2 + '\n').encode('utf-8'),
         protocol=None
     )
 
 
-@timed(3)
+@pytest.mark.fail_slow(3)
 def test_runner_no_stdin_no_capture():
     # Ensure a runner without stdin data and output capture progresses
     runner = Runner()
@@ -177,7 +170,7 @@ def test_runner_no_stdin_no_capture():
     )
 
 
-@timed(3)
+@pytest.mark.fail_slow(3)
 def test_runner_empty_stdin():
     # Ensure a runner without stdin data and output capture progresses
     runner = Runner()
@@ -212,7 +205,7 @@ def test_runner_parametrized_protocol():
 @integration  # ~3 sec
 @with_tempfile(mkdir=True)
 @with_tempfile()
-def test_asyncio_loop_noninterference1(path1, path2):
+def test_asyncio_loop_noninterference1(path1=None, path2=None):
     if on_windows and sys.version_info < (3, 8):
         raise SkipTest(
             "get_event_loop() raises "
@@ -237,7 +230,7 @@ ds.status()
 
 
 @with_tempfile
-def test_asyncio_forked(temp):
+def test_asyncio_forked(temp=None):
     # temp will be used to communicate from child either it succeeded or not
     temp = Path(temp)
     runner = Runner()
@@ -300,6 +293,15 @@ def test_faulty_poll_detection():
     assert_raises(CommandError, protocol._prepare_result)
 
 
+def test_kill_output():
+    runner = Runner()
+    res = runner.run(
+        py2cmd('import sys; sys.stdout.write("aaaa\\n"); sys.stderr.write("bbbb\\n")'),
+        protocol=KillOutput)
+    eq_(res['stdout'], '')
+    eq_(res['stderr'], '')
+
+
 @skip_if_on_windows  # no "hint" on windows since no ulimit command there
 def test_too_long():
     with swallow_logs(new_level=logging.ERROR) as cml:
@@ -308,4 +310,4 @@ def test_too_long():
                 [sys.executable, '-c', 'import sys; print(len(sys.argv))'] + [str(i) for i in range(CMD_MAX_ARG)],
                 protocol=StdOutCapture
             )
-        cml.assert_logged('.*use.*ulimit.*')
\ No newline at end of file
+        cml.assert_logged('.*use.*ulimit.*')
