@@ -1,6 +1,6 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
 # -*- coding: utf-8 -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -11,14 +11,16 @@
 
 """
 
+import inspect
+import logging
 import os
 import os.path as op
 import shutil
+import stat
 import sys
 import time
-import logging
-from unittest.mock import patch
-
+from collections import OrderedDict
+from functools import wraps
 from operator import itemgetter
 from os.path import (
     abspath,
@@ -28,34 +30,43 @@ from os.path import (
     expanduser,
     expandvars,
     isabs,
-    join as opj,
+)
+from os.path import join as opj
+from os.path import (
     normpath,
     pardir,
 )
-from collections import OrderedDict
+from unittest.mock import patch
 
+import pytest
+
+from datalad import cfg as dl_cfg
+from datalad.support.annexrepo import AnnexRepo
 from datalad.utils import (
+    CMD_MAX_ARG,
+    Path,
     _path_,
     all_same,
     any_re_search,
     auto_repr,
     better_wraps,
-    CMD_MAX_ARG,
-    Path,
+    chpwd,
     create_tree,
     disable_logger,
     dlabspath,
+    ensure_write_permission,
     expandpath,
     file_basename,
     find_files,
     generate_chunks,
     get_dataset_root,
-    get_func_kwargs_doc,
     get_open_files,
     get_path_prefix,
+    get_sig_param_names,
     get_timestamp_suffix,
     get_trace,
-    getpwd, chpwd,
+    getargspec,
+    getpwd,
     import_module_from_file,
     import_modules,
     is_explicit_path,
@@ -68,22 +79,24 @@ from datalad.utils import (
     md5sum,
     never_fail,
     not_supported_on_windows,
+    obtain_write_permission,
     on_windows,
     partition,
     path_is_subpath,
     path_startswith,
     rotree,
-    setup_exceptionhook,
     split_cmdline,
     swallow_logs,
     swallow_outputs,
+    todo_interface_for_extensions,
     unique,
     unlink,
     updated,
 )
-from datalad.support.annexrepo import AnnexRepo
 
-from .utils import (
+from .utils_pytest import (
+    OBSCURE_FILENAME,
+    SkipTest,
     as_unicode,
     assert_cwd_unchanged,
     assert_equal,
@@ -103,7 +116,6 @@ from .utils import (
     has_symlink_capability,
     known_failure,
     nok_,
-    OBSCURE_FILENAME,
     ok_,
     ok_file_has_content,
     ok_generator,
@@ -113,25 +125,15 @@ from .utils import (
     skip_if,
     skip_if_no_module,
     skip_if_on_windows,
+    skip_if_root,
     skip_known_failure,
-    SkipTest,
     skip_wo_symlink_capability,
     with_tempfile,
     with_tree,
 )
-from datalad import cfg as dl_cfg
-
-
-def test_get_func_kwargs_doc():
-    def some_func(arg1, kwarg1=None, kwarg2="bu"):
-        return
-    eq_(get_func_kwargs_doc(some_func), ['arg1', 'kwarg1', 'kwarg2'])
 
 
 def test_better_wraps():
-    from functools import wraps
-    from datalad.utils import getargspec
-
     def wraps_decorator(func):
         @wraps(func)
         def  _wrap_wraps_decorator(*args, **kwargs):
@@ -155,13 +157,84 @@ def test_better_wraps():
         return "function2"
 
     eq_("function1", function1(1, 2, 3))
-    eq_(getargspec(function1)[0], [])
+    # getargspec shim now can handle @wraps'ed functions just fine
+    eq_(getargspec(function1)[0], ['a', 'b', 'c'])
     eq_("function2", function2(1, 2, 3))
     eq_(getargspec(function2)[0], ['a', 'b', 'c'])
 
 
+# TODO?: make again parametric on eq_argspec invocations?
+@pytest.mark.filterwarnings(r"ignore: inspect.getargspec\(\) is deprecated")
+def test_getargspec():
+
+    def eq_argspec(f, expected, has_kwonlyargs=False):
+        """A helper to centralize testing of getargspec on original and wrapped function
+
+        has_kwonlyargs is to instruct if function has kwonly args so we do not try to compare
+        to inspect.get*spec functions, which would barf ValueError if attempted to run on a
+        function with kwonlys. And also we pass it as include_kwonlyargs to our getargspec
+        """
+        # so we know that our expected is correct
+        if not has_kwonlyargs:
+            # if False - we test function with kwonlys - inspect.getargspec would barf
+            eq_(inspect.getargspec(f), expected)
+            # and getfullargspec[:4] wouldn't provide a full picture
+            eq_(inspect.getfullargspec(f)[:4], expected)
+        else:
+            assert_raises(ValueError, inspect.getargspec, f)
+            inspect.getfullargspec(f)  # doesn't barf
+        eq_(getargspec(f, include_kwonlyargs=has_kwonlyargs), expected)
+
+        # and lets try on a wrapped one -- only ours can do the right thing
+        def decorator(f):
+            @wraps(f)
+            def wrapper(*args, **kwargs):  # pragma: no cover
+                return f(*args, **kwargs)
+            return wrapper
+        fw = decorator(f)
+        if has_kwonlyargs:
+            # We barf ValueError similarly to inspect.getargspec, unless explicitly requested
+            # to include kwonlyargs
+            assert_raises(ValueError, getargspec, fw)
+        eq_(getargspec(fw, include_kwonlyargs=has_kwonlyargs), expected)
+
+    def f0():  # pragma: no cover
+        pass
+
+    eq_argspec(f0, ([], None, None, None))
+
+    def f1(a1, kw1=None, kw0=1):  # pragma: no cover
+        pass
+
+    eq_argspec(f1, (['a1', 'kw1', 'kw0'], None, None, (None, 1)))
+
+    # Having *a already makes keyword args to be kwonlyargs, in that
+    # inspect.get*spec would barf
+    def f1_args(a1, *a, kw1=None, kw0=1, **kw):  # pragma: no cover
+        pass
+
+    eq_argspec(f1_args, (['a1', 'kw1', 'kw0'], 'a', 'kw', (None, 1)), True)
+
+    def f1_star(a1, *, kw1=None, kw0=1):  # pragma: no cover
+        pass
+
+    assert_raises(ValueError, getargspec, f1_star)
+    eq_argspec(f1_star, (['a1', 'kw1', 'kw0'], None, None, (None, 1)), True)
+
+
+def test_get_sig_param_names():
+    def f(a1, kw1=None, *args, kw2=None, **kwargs):
+        pass  # pragma: no cover
+
+    # note: `a1` could be used either positionally or via keyword, so is listed in kw_any
+    assert_equal(get_sig_param_names(f, ('kw_only', 'kw_any')), (['kw2'], ['a1', 'kw1', 'kw2']))
+    assert_equal(get_sig_param_names(f, ('any',)), (['a1', 'kw1', 'kw2'],))
+    assert_equal(get_sig_param_names(f, tuple()), ())
+    assert_raises(ValueError, get_sig_param_names, f, ('mumba',))
+
+
 @with_tempfile(mkdir=True)
-def test_rotree(d):
+def test_rotree(d=None):
     d2 = opj(d, 'd1', 'd2')  # deep nested directory
     f = opj(d2, 'f1')
     os.makedirs(d2)
@@ -202,7 +275,7 @@ def test_swallow_outputs():
 
 
 @with_tempfile
-def test_swallow_logs(logfile):
+def test_swallow_logs(logfile=None):
     lgr = logging.getLogger('datalad')
     with swallow_logs(new_level=9) as cm:
         eq_(cm.out, '')
@@ -216,7 +289,7 @@ def test_swallow_logs(logfile):
     with swallow_logs(new_level=9, file_=logfile) as cm:
         eq_(cm.out, '')
         lgr.info("next info")
-    from datalad.tests.utils import ok_file_has_content
+    from datalad.tests.utils_pytest import ok_file_has_content
     ok_file_has_content(logfile, "[INFO] next info", strip=True)
 
 
@@ -257,7 +330,7 @@ def test_disable_logger():
     # get a logger hierarchy:
     lgr_top = logging.getLogger('datalad')
     lgr_middle = logging.getLogger('datalad.tests')
-    lgr_bottom = logging.getLogger('datalad.tests.utils')
+    lgr_bottom = logging.getLogger('datalad.tests.utils_pytest')
 
     with swallow_logs(new_level=logging.DEBUG) as cml:
         with disable_logger():  # default: 'datalad':
@@ -280,58 +353,13 @@ def test_disable_logger():
         assert_raises(AssertionError, cml.assert_logged, "log sth at bottom level")
 
 
-def _check_setup_exceptionhook(interactive):
-    old_exceptionhook = sys.excepthook
-
-    post_mortem_tb = []
-
-    def our_post_mortem(tb):
-        post_mortem_tb.append(tb)
-
-    with patch('sys.excepthook'), \
-            patch('datalad.utils.is_interactive', lambda: interactive), \
-            patch('pdb.post_mortem', our_post_mortem):
-        setup_exceptionhook()
-        our_exceptionhook = sys.excepthook
-        ok_(old_exceptionhook != our_exceptionhook)
-        with swallow_logs() as cml, swallow_outputs() as cmo:
-            # we need to call our_exceptionhook explicitly b/c nose
-            # swallows all Exceptions and hook never gets executed
-            try:
-                raise RuntimeError
-            except Exception as e:  # RuntimeError:
-                type_, value_, tb_ = sys.exc_info()
-            our_exceptionhook(type_, value_, tb_)
-            # Happens under tox environment but not in manually crafted
-            # ones -- not yet sure what it is about but --dbg does work
-            # with python3 so lettting it skip for now
-            raise SkipTest(
-                "TODO: Not clear why in PY3 calls cleanup if we try to "
-                "access the beast"
-            )
-            #assert_in('Traceback (most recent call last)', cmo.err)
-            #assert_in('in _check_setup_exceptionhook', cmo.err)
-            #if interactive:
-            #    assert_equal(post_mortem_tb[0], tb_)
-            #else:
-            #    assert_equal(post_mortem_tb, [])
-            #    # assert_in('We cannot setup exception hook', cml.out)
-
-    eq_(old_exceptionhook, sys.excepthook)
-
-
-def test_setup_exceptionhook():
-    for tval in [True, False]:
-        yield _check_setup_exceptionhook, tval
-
-
 def test_md5sum():
     # just a smoke (encoding/decoding) test for md5sum
     _ = md5sum(__file__)
 
 
 @with_tree([('1.tar.gz', (('1 f.txt', '1 f load'),))])
-def test_md5sum_archive(d):
+def test_md5sum_archive(d=None):
     # just a smoke (encoding/decoding) test for md5sum
     _ = md5sum(opj(d, '1.tar.gz'))
 
@@ -371,7 +399,7 @@ def test_getpwd_basic():
 
 @with_tempfile(mkdir=True)
 @assert_cwd_unchanged(ok_to_chdir=True)
-def test_getpwd_change_mode(tdir):
+def test_getpwd_change_mode(tdir=None):
     from datalad import utils
     if utils._pwd_mode != 'PWD':
         raise SkipTest("Makes sense to be tested only in PWD mode, "
@@ -390,7 +418,7 @@ def test_getpwd_change_mode(tdir):
 @skip_if_on_windows
 @with_tempfile(mkdir=True)
 @assert_cwd_unchanged
-def test_getpwd_symlink(tdir):
+def test_getpwd_symlink(tdir=None):
     sdir = opj(tdir, 's1')
     pwd_orig = getpwd()
     Path(sdir).symlink_to(Path('.'))
@@ -424,7 +452,7 @@ def test_getpwd_symlink(tdir):
 
 
 @with_tempfile(mkdir=True)
-def test_chpwd_obscure_name(topdir):
+def test_chpwd_obscure_name(topdir=None):
     path = op.join(topdir, OBSCURE_FILENAME)
     os.mkdir(path)
     # Just check that call doesn't fail.
@@ -442,19 +470,43 @@ def test_auto_repr():
     class buga:
         def __init__(self):
             self.a = 1
-            self.b = list(range(100))
+            self.b = list(range(20))
             self.c = WithoutReprClass()
             self._c = "protect me"
 
         def some(self):
             return "some"
 
+    @auto_repr(short=False)
+    class buga_long(object):
+        def __init__(self):
+            self.a = 1
+            self.b = list(range(20))
+
+        def some(self):
+            return "some"
+
     assert_equal(
         repr(buga()),
-        "buga(a=1, b=<<[0, 1, 2, 3, 4++372 chars++ 99]>>, c=<WithoutReprClass>)"
+        "buga(a=1, b=<<[0, 1, 2, 3, 4++52 chars++ 19]>>, c=<WithoutReprClass>)"
     )
     assert_equal(buga().some(), "some")
 
+    assert_equal(
+        repr(buga_long()),
+        f"buga_long(a=1, b=[{', '.join(map(str, range(20)))}])"
+    )
+    assert_equal(buga_long().some(), "some")
+
+
+def test_todo_interface_for_extensions():
+
+    @todo_interface_for_extensions
+    def f(i, j):
+        return i*j
+
+    assert_equal(f(2, 3), 6)
+
 
 def test_assure_iter():
     s = {1}
@@ -471,13 +523,19 @@ def test_assure_list_copy():
     assert ensure_list(l, copy=True) is not l
 
 
-def test_assure_list_from_str():
-    assert_equal(ensure_list_from_str(''), None)
-    assert_equal(ensure_list_from_str([]), None)
-    assert_equal(ensure_list_from_str('somestring'), ['somestring'])
-    assert_equal(ensure_list_from_str('some\nmultiline\nstring'), ['some', 'multiline', 'string'])
-    assert_equal(ensure_list_from_str(['something']), ['something'])
-    assert_equal(ensure_list_from_str(['a', 'listof', 'stuff']), ['a', 'listof', 'stuff'])
+@pytest.mark.parametrize(
+    "value,result",
+    [
+        ('', None),
+        ([], None),
+        ('somestring', ['somestring']),
+        ('some\nmultiline\nstring', ['some', 'multiline', 'string']),
+        (['something'], ['something']),
+        (['a', 'listof', 'stuff'], ['a', 'listof', 'stuff']),
+    ]
+)
+def test_assure_list_from_str(value, result):
+    assert ensure_list_from_str(value) == result
 
 
 def test_assure_dict_from_str():
@@ -562,7 +620,7 @@ def test_find_files():
     },
     'git': 'just a file'
 })
-def test_find_files_exclude_vcs(repo):
+def test_find_files_exclude_vcs(repo=None):
     ff = find_files('.*', repo, dirs=True)
     files = list(ff)
     assert_equal({basename(f) for f in files}, {'d1', 'git'})
@@ -618,9 +676,9 @@ def test_is_explicit_path():
 
 @with_tempfile
 @with_tempfile
-def test_knows_annex(here, there):
-    from datalad.support.gitrepo import GitRepo
+def test_knows_annex(here=None, there=None):
     from datalad.support.annexrepo import AnnexRepo
+    from datalad.support.gitrepo import GitRepo
     GitRepo(path=here, create=True)
     assert_false(knows_annex(here))
     AnnexRepo(path=here, create=True)
@@ -668,7 +726,7 @@ def test_all_same():
         yield 'a'
         yield 'a'
         yield 'b'
-        raise ValueError("Should not get here since on b should return")
+        raise ValueError("Should not get here since on b should return")  # pragma: no cover
 
     ok_(not all_same(never_get_to_not_needed()))
 
@@ -794,12 +852,12 @@ def test_as_unicode():
     eq_(as_unicode("01"), u"01")  # no some kind of conversion/stripping of numerals
     with assert_raises(TypeError) as cme:
         as_unicode(1, list)
-    assert_in("1 is not of any of known or provided", str(cme.exception))
+    assert_in("1 is not of any of known or provided", str(cme.value))
 
 
 @skip_if_on_windows
 @with_tempfile(mkdir=True)
-def test_path_prefix(path):
+def test_path_prefix(path=None):
     eq_(get_path_prefix('/d1/d2', '/d1/d2'), '')
     # so we are under /d1/d2 so path prefix is ..
     eq_(get_path_prefix('/d1/d2', '/d1/d2/d3'), '..')
@@ -843,7 +901,7 @@ def test_get_trace():
 
 
 @with_tempfile(mkdir=True)
-def test_get_dataset_root(path):
+def test_get_dataset_root(path=None):
     eq_(get_dataset_root('/nonexistent'), None)
     with chpwd(path):
         repo = AnnexRepo(os.curdir, create=True)
@@ -991,7 +1049,7 @@ from datalad.utils import read_csv_lines
 
 def test_known_failure_direct_mode():
     # Decorator is deprecated now and that is what we check
-    from .utils import known_failure_direct_mode
+    from .utils_pytest import known_failure_direct_mode
 
     x = []
     with swallow_logs(new_level=logging.WARNING) as cml:
@@ -1006,7 +1064,7 @@ def test_known_failure_direct_mode():
 
 
 @with_tempfile(content="h1 h2\nv1 2\nv2 3")
-def test_read_csv_lines_basic(infile):
+def test_read_csv_lines_basic(infile=None):
     # Just a basic test, next one with unicode
     gen = read_csv_lines(infile)
     ok_generator(gen)
@@ -1020,7 +1078,7 @@ def test_read_csv_lines_basic(infile):
 
 
 @with_tempfile(content=u"h1\th2\nv1\tдата".encode('utf-8'))
-def test_read_csv_lines_tsv_unicode(infile):
+def test_read_csv_lines_tsv_unicode(infile=None):
     # Just a basic test, next one with unicode
     gen = read_csv_lines(infile)
     ok_generator(gen)
@@ -1033,7 +1091,7 @@ def test_read_csv_lines_tsv_unicode(infile):
 
 
 @with_tempfile(content=u"h1\nv1\nv2")
-def test_read_csv_lines_one_column(infile):
+def test_read_csv_lines_one_column(infile=None):
     # Just a basic test, next one with unicode
     eq_(
         list(read_csv_lines(infile)),
@@ -1059,7 +1117,7 @@ def _get_testm_tree(ind):
     }
 
 @with_tree(tree=_get_testm_tree(1))
-def test_import_modules(topdir):
+def test_import_modules(topdir=None):
     try:
         sys.path.append(topdir)
         mods = import_modules(['dlsub1', 'bogus'], 'dltestm1')
@@ -1070,7 +1128,7 @@ def test_import_modules(topdir):
 
 
 @with_tree(tree=_get_testm_tree(2))
-def test_import_module_from_file(topdir):
+def test_import_module_from_file(topdir=None):
     with assert_raises(AssertionError):
         # we support only submodule files ending with .py ATM. TODO
         import_module_from_file(op.join(topdir, 'dltestm2', 'dlsub1'))
@@ -1114,7 +1172,7 @@ def test_line_profile():
 
 
 @with_tempfile(mkdir=True)
-def test_dlabspath(path):
+def test_dlabspath(path=None):
     if not has_symlink_capability():
         raise SkipTest
     # initially ran into on OSX https://github.com/datalad/datalad/issues/2406
@@ -1134,7 +1192,7 @@ def test_dlabspath(path):
 
 
 @with_tree({'1': 'content', 'd': {'2': 'more'}})
-def test_get_open_files(p):
+def test_get_open_files(p=None):
     pobj = Path(p)
     skip_if_no_module('psutil')
     eq_(get_open_files(p), {})
@@ -1159,7 +1217,10 @@ def test_get_open_files(p):
         return
 
     # if we start a process within that directory, should get informed
-    from subprocess import Popen, PIPE
+    from subprocess import (
+        PIPE,
+        Popen,
+    )
     from time import time
     t0 = time()
     proc = Popen([sys.executable, '-c',
@@ -1203,7 +1264,7 @@ def test_CMD_MAX_ARG():
 
 
 @with_tempfile(mkdir=True)
-def test_create_tree(path):
+def test_create_tree(path=None):
     content = u"мама мыла раму"
     create_tree(path, OrderedDict([
         ('1', content),
@@ -1246,8 +1307,9 @@ def test_never_fail():
         assert_raises(ValueError, ifail2, 1)
 
 
+@pytest.mark.xfail(reason="TODO: for some reason fails on Travis")
 @with_tempfile
-def test_is_interactive(fout):
+def test_is_interactive(fout=None):
     # must not fail if one of the streams is no longer open:
     # https://github.com/datalad/datalad/issues/3267
     from datalad.cmd import (
@@ -1256,11 +1318,11 @@ def test_is_interactive(fout):
         StdOutErrCapture,
         WitlessRunner,
     )
-    from datalad.support.gitrepo import GitProgress
     from datalad.support.annexrepo import (
         AnnexInitOutput,
         AnnexJsonProtocol,
     )
+    from datalad.support.gitrepo import GitProgress
 
     bools = ["False", "True"]
 
@@ -1326,3 +1388,77 @@ def test_splitjoin_cmdline():
         eq_(join_cmdline(['abc', 'def']), '"abc" "def"')
     else:
         eq_(join_cmdline(['abc', 'def']), 'abc def')
+
+
+@skip_if_root
+@with_tempfile
+def test_obtain_write_permission(path=None):
+    path = Path(path)
+
+    # there's nothing at path yet:
+    assert_raises(FileNotFoundError, obtain_write_permission, path)
+
+    # Revoke write permission
+    path.write_text("something")
+    path.chmod(path.stat().st_mode & ~stat.S_IWRITE)
+    assert_raises(PermissionError, path.write_text, "different thing")
+
+    # Obtain and try again:
+    obtain_write_permission(path)
+    path.write_text("different thing")
+
+    # Already having permission is no issue:
+    obtain_write_permission(path)
+    path.write_text("yet another thing")
+
+
+@skip_if_root
+@with_tempfile(mkdir=True)
+def test_ensure_write_permission(path=None):
+
+    # This is testing the usecase of write protected directories needed for
+    # messing with an annex object tree (as done by the ORA special remote).
+    # However, that doesn't work on Windows since we can't revoke write
+    # permissions for the owner of a directory (at least on VFAT - may be
+    # true for NTFS as well - don't know).
+    # Hence, on windows/crippledFS only test on a file.
+
+    dir_ = Path(path)
+    if not on_windows and has_symlink_capability:
+        # set up write-protected dir containing a file
+        file_ = dir_ / "somefile"
+        file_.write_text("whatever")
+        dir_.chmod(dir_.stat().st_mode & ~stat.S_IWRITE)
+        assert_raises(PermissionError, file_.unlink)
+
+        # contextmanager lets us do it and restores permissions afterwards:
+        mode_before = dir_.stat().st_mode
+        with ensure_write_permission(dir_):
+            file_.unlink()
+
+        mode_after = dir_.stat().st_mode
+        assert_equal(mode_before, mode_after)
+        assert_raises(PermissionError, file_.write_text, "new file can't be "
+                                                         "written")
+
+        assert_raises(FileNotFoundError, ensure_write_permission(dir_ /
+                      "non" / "existent").__enter__)
+
+        # deletion within context doesn't let mode restoration fail:
+        with ensure_write_permission(dir_):
+            dir_.rmdir()
+
+        dir_.mkdir()  # recreate, since next block is executed unconditionally
+
+    # set up write-protected file:
+    file2 = dir_ / "protected.txt"
+    file2.write_text("unchangeable")
+    file2.chmod(file2.stat().st_mode & ~stat.S_IWRITE)
+    assert_raises(PermissionError, file2.write_text, "modification")
+
+    # within context we can:
+    with ensure_write_permission(file2):
+        file2.write_text("modification")
+
+    # mode is restored afterwards:
+    assert_raises(PermissionError, file2.write_text, "modification2")
