@@ -12,51 +12,53 @@ import base64
 import glob
 import gzip
 import inspect
+import logging
 import lzma
-import shutil
-import stat
-from json import dumps
-import os
-import re
-import tempfile
-import platform
 import multiprocessing
 import multiprocessing.queues
-import logging
+import os
+import platform
 import random
+import re
+import shutil
 import socket
-import textwrap
-import warnings
-from fnmatch import fnmatch
-import time
-from difflib import unified_diff
-from contextlib import contextmanager
-from unittest.mock import patch
 import ssl
-
+import stat
+import tempfile
+import textwrap
+import time
+import warnings
+from contextlib import contextmanager
+from difflib import unified_diff
+from fnmatch import fnmatch
+from functools import wraps
 from http.server import (
     HTTPServer,
     SimpleHTTPRequestHandler,
 )
-
-from functools import wraps
+from json import dumps
 from os.path import (
     curdir,
     exists,
-    join as opj,
-    relpath,
-    split as pathsplit,
 )
+from os.path import join as opj
+from os.path import relpath
+from os.path import split as pathsplit
+from unittest.mock import patch
 
+from nose import SkipTest
 from nose.plugins.attrib import attr
 from nose.tools import (
     assert_equal,
     assert_false,
     assert_greater,
     assert_greater_equal,
-    assert_in as in_,
-    assert_in,
+)
+from nose.tools import assert_in
+from nose.tools import assert_in as in_
+from nose.tools import (
     assert_is,
+    assert_is_instance,
     assert_is_none,
     assert_is_not,
     assert_is_not_none,
@@ -64,6 +66,7 @@ from nose.tools import (
     assert_not_in,
     assert_not_is_instance,
     assert_raises,
+    assert_set_equal,
     assert_true,
     eq_,
     make_decorator,
@@ -71,33 +74,12 @@ from nose.tools import (
     raises,
 )
 
-from nose.tools import assert_set_equal
-from nose.tools import assert_is_instance
-from nose import SkipTest
-
-from datalad import cfg as dl_cfg
 import datalad.utils as ut
-# TODO this must go
-from ..utils import *
-from datalad.utils import (
-    Path,
-    ensure_unicode,
+from datalad import cfg as dl_cfg
+from datalad import (
+    conftest,
+    ssh_manager,
 )
-
-from .. import utils
-from ..support.exceptions import (
-    CommandError,
-    CommandNotAvailableError,
-)
-from ..support.external_versions import external_versions
-from ..support.vcr_ import *
-from ..support.keyring_ import MemoryKeyring
-from ..support.network import RI
-from ..dochelpers import borrowkwargs
-from ..consts import (
-    ARCHIVES_TEMP_DIR,
-)
-from . import _TEMP_PATHS_GENERATED
 from datalad.cmd import (
     GitWitlessRunner,
     KillOutput,
@@ -105,7 +87,28 @@ from datalad.cmd import (
     WitlessRunner,
 )
 from datalad.core.local.repo import repo_from_path
+from datalad.utils import (
+    Path,
+    ensure_unicode,
+    get_encoding_info,
+    get_envvars_info,
+    get_home_envvars,
+)
 
+from .. import utils
+from ..consts import ARCHIVES_TEMP_DIR
+from ..dochelpers import borrowkwargs
+from ..support.exceptions import (
+    CommandError,
+    CommandNotAvailableError,
+)
+from ..support.external_versions import external_versions
+from ..support.keyring_ import MemoryKeyring
+from ..support.network import RI
+from ..support.vcr_ import *
+# TODO this must go
+from ..utils import *
+from . import _TEMP_PATHS_GENERATED
 
 # temp paths used by clones
 _TEMP_PATHS_CLONES = set()
@@ -134,6 +137,247 @@ nok_ = assert_false
 
 lgr = logging.getLogger("datalad.tests.utils")
 
+# To store settings which setup_package changes and teardown_package should return
+_test_states = {
+    'loglevel': None,
+    'env': {},
+}
+
+def setup_package():
+    import os
+    import tempfile
+    from io import StringIO as OrigStringIO
+    from pathlib import Path
+
+    from nose.ext import dtcompat
+    from nose.plugins import (
+        capture,
+        multiprocess,
+        plugintest,
+    )
+
+    from datalad import consts, lgr
+    from datalad.support.annexrepo import AnnexRepo
+    from datalad.support.external_versions import external_versions
+    from datalad.tests.utils import (
+        DEFAULT_BRANCH,
+        DEFAULT_REMOTE,
+        HTTPPath,
+    )
+    from datalad.ui import ui
+    from datalad.utils import (
+        make_tempfile,
+        on_osx,
+    )
+
+    if on_osx:
+        # enforce honoring TMPDIR (see gh-5307)
+        tempfile.tempdir = os.environ.get('TMPDIR', tempfile.gettempdir())
+
+    _test_states['env'] = {}
+
+    def set_envvar(v, val):
+        """Memoize and then set env var"""
+        _test_states['env'][v] = os.environ.get(v, None)
+        os.environ[v] = val
+
+    _test_states['DATASETS_TOPURL'] = consts.DATASETS_TOPURL
+    consts.DATASETS_TOPURL = 'https://datasets-tests.datalad.org/'
+    set_envvar('DATALAD_DATASETS_TOPURL', consts.DATASETS_TOPURL)
+
+    set_envvar("GIT_CONFIG_PARAMETERS",
+               "'init.defaultBranch={}' 'clone.defaultRemoteName={}'"
+               .format(DEFAULT_BRANCH, DEFAULT_REMOTE))
+
+    def prep_tmphome():
+        # re core.askPass:
+        # Don't let git ask for credentials in CI runs. Note, that this variable
+        # technically is not a flag, but an executable (which is why name and value
+        # are a bit confusing here - we just want a no-op basically). The environment
+        # variable GIT_ASKPASS overwrites this, but neither env var nor this config
+        # are supported by git-credential on all systems and git versions (most recent
+        # ones should work either way, though). Hence use both across CI builds.
+        gitconfig = """\
+[user]
+	name = DataLad Tester
+	email = test@example.com
+[core]
+	askPass =
+[datalad "log"]
+	exc = 1
+[annex "security"]
+	# from annex 6.20180626 file:/// and http://localhost access isn't
+	# allowed by default
+	allowed-url-schemes = http https file
+	allowed-http-addresses = all
+"""
+        # TODO: split into a function + context manager
+        with make_tempfile(mkdir=True) as new_home:
+            pass
+        # register for clean-up on exit
+        _TEMP_PATHS_GENERATED.append(new_home)
+
+        # populate default config
+        new_home = Path(new_home)
+        new_home.mkdir(parents=True, exist_ok=True)
+        cfg_file = new_home / '.gitconfig'
+        cfg_file.write_text(gitconfig)
+        return new_home, cfg_file
+
+    if external_versions['cmd:git'] < "2.32":
+        # To overcome pybuild overriding HOME but us possibly wanting our
+        # own HOME where we pre-setup git for testing (name, email)
+        if 'GIT_HOME' in os.environ:
+            set_envvar('HOME', os.environ['GIT_HOME'])
+        else:
+            # we setup our own new HOME, the BEST and HUGE one
+            new_home, _ = prep_tmphome()
+            for v, val in get_home_envvars(new_home).items():
+                set_envvar(v, val)
+    else:
+        _, cfg_file = prep_tmphome()
+        set_envvar('GIT_CONFIG_GLOBAL', str(cfg_file))
+
+    # Re-load ConfigManager, since otherwise it won't consider global config
+    # from new $HOME (see gh-4153
+    dl_cfg.reload(force=True)
+
+    # datalad.locations.sockets has likely changed. Discard any cached values.
+    ssh_manager._socket_dir = None
+
+    # To overcome pybuild by default defining http{,s}_proxy we would need
+    # to define them to e.g. empty value so it wouldn't bother touching them.
+    # But then haskell libraries do not digest empty value nicely, so we just
+    # pop them out from the environment
+    for ev in ('http_proxy', 'https_proxy'):
+        if ev in os.environ and not (os.environ[ev]):
+            lgr.debug("Removing %s from the environment since it is empty", ev)
+            os.environ.pop(ev)
+
+    DATALAD_LOG_LEVEL = os.environ.get('DATALAD_LOG_LEVEL', None)
+    if DATALAD_LOG_LEVEL is None:
+        # very very silent.  Tests introspecting logs should use
+        # swallow_logs(new_level=...)
+        _test_states['loglevel'] = lgr.getEffectiveLevel()
+        lgr.setLevel(100)
+
+        # And we should also set it within environ so underlying commands also stay silent
+        set_envvar('DATALAD_LOG_LEVEL', '100')
+    else:
+        # We are not overriding them, since explicitly were asked to have some log level
+        _test_states['loglevel'] = None
+
+    # Prevent interactive credential entry (note "true" is the command to run)
+    # See also the core.askPass setting above
+    set_envvar('GIT_ASKPASS', 'true')
+
+    # Set to non-interactive UI
+    _test_states['ui_backend'] = ui.backend
+    # obtain() since that one consults for the default value
+    ui.set_backend(dl_cfg.obtain('datalad.tests.ui.backend'))
+
+    # Monkey patch nose so it does not ERROR out whenever code asks for fileno
+    # of the output. See https://github.com/nose-devs/nose/issues/6
+    class StringIO(OrigStringIO):
+        fileno = lambda self: 1
+        encoding = None
+
+    dtcompat.StringIO = StringIO
+    capture.StringIO = StringIO
+    multiprocess.StringIO = StringIO
+    plugintest.StringIO = StringIO
+
+    # in order to avoid having to fiddle with rather uncommon
+    # file:// URLs in the tests, have a standard HTTP server
+    # that serves an 'httpserve' directory in the test HOME
+    # the URL will be available from datalad.test_http_server.url
+    # Start the server only if not running already
+    # Relevant: we have test_misc.py:test_test which runs datalad.test but
+    # not doing teardown, so the original server might never get stopped
+    if conftest.test_http_server is None:
+        serve_path = tempfile.mkdtemp(
+            dir=dl_cfg.get("datalad.tests.temp.dir"),
+            prefix='httpserve',
+        )
+        conftest.test_http_server = HTTPPath(serve_path)
+        conftest.test_http_server.start()
+        _TEMP_PATHS_GENERATED.append(serve_path)
+
+    if dl_cfg.obtain('datalad.tests.setup.testrepos'):
+        lgr.debug("Pre-populating testrepos")
+        from datalad.tests.utils import with_testrepos
+        with_testrepos()(lambda repo: 1)()
+
+
+def teardown_package():
+    import os
+
+    from datalad import consts
+    from datalad.support.annexrepo import AnnexRepo
+    from datalad.support.cookies import cookies_db
+    from datalad.support.external_versions import external_versions as ev
+    from datalad.tests.utils import (
+        OBSCURE_FILENAME,
+        rmtemp,
+    )
+    from datalad.ui import ui
+
+    lgr.debug("Printing versioning information collected so far")
+    # Query for version of datalad, so it is included in ev.dumps below - useful while
+    # testing extensions where version of datalad might differ in the environment.
+    ev['datalad']
+    print(ev.dumps(query=True))
+    try:
+        print("Obscure filename: str=%s repr=%r"
+                % (OBSCURE_FILENAME.encode('utf-8'), OBSCURE_FILENAME))
+    except UnicodeEncodeError as exc:
+        ce = CapturedException(exc)
+        print("Obscure filename failed to print: %s" % ce)
+    def print_dict(d):
+        return " ".join("%s=%r" % v for v in d.items())
+    print("Encodings: %s" % print_dict(get_encoding_info()))
+    print("Environment: %s" % print_dict(get_envvars_info()))
+
+    if os.environ.get('DATALAD_TESTS_NOTEARDOWN'):
+        return
+    ui.set_backend(_test_states['ui_backend'])
+    if _test_states['loglevel'] is not None:
+        lgr.setLevel(_test_states['loglevel'])
+
+    if conftest.test_http_server:
+        conftest.test_http_server.stop()
+        conftest.test_http_server = None
+    else:
+        lgr.debug("For some reason global http_server was not set/running, thus not stopping")
+
+    if len(_TEMP_PATHS_GENERATED):
+        msg = "Removing %d dirs/files: %s" % (len(_TEMP_PATHS_GENERATED), ', '.join(_TEMP_PATHS_GENERATED))
+    else:
+        msg = "Nothing to remove"
+    lgr.debug("Teardown tests. " + msg)
+    for path in _TEMP_PATHS_GENERATED:
+        rmtemp(str(path), ignore_errors=True)
+
+    # restore all the env variables
+    for v, val in _test_states['env'].items():
+        if val is not None:
+            os.environ[v] = val
+        else:
+            os.environ.pop(v)
+
+    # Re-establish correct global config after changing $HOME.
+    # Might be superfluous, since after teardown datalad.cfg shouldn't be
+    # needed. However, maintaining a consistent state seems a good thing
+    # either way.
+    dl_cfg.reload(force=True)
+
+    ssh_manager._socket_dir = None
+
+    consts.DATASETS_TOPURL = _test_states['DATASETS_TOPURL']
+
+    cookies_db.close()
+    AnnexRepo._ALLOW_LOCAL_URLS = False  # stay safe!
+
 
 def skip_if_no_module(module):
     try:
@@ -156,8 +400,8 @@ def skip_if_scrapy_without_selector():
 
 def skip_if_url_is_not_available(url, regex=None):
     # verify that dataset is available
-    from datalad.downloaders.providers import Providers
     from datalad.downloaders.base import DownloadError
+    from datalad.downloaders.providers import Providers
     providers = Providers.from_config_files()
     try:
         content = providers.fetch(url)
@@ -273,8 +517,8 @@ def skip_if(func, cond=True, msg=None, method='raise'):
 
 
 def skip_ssh(func):
-    """Skips SSH tests if on windows or if environment variable
-    DATALAD_TESTS_SSH was not set
+    """Skips SSH tests if datalad.tests.ssh (env var DATALAD_TESTS_SSH)
+    is not set at all or set to '0', 'false', or 'no'.
     """
 
     check_not_generatorfunction(func)
@@ -296,7 +540,10 @@ def skip_nomultiplex_ssh(func):
     """
 
     check_not_generatorfunction(func)
-    from ..support.sshconnector import MultiplexSSHManager, SSHManager
+    from ..support.sshconnector import (
+        MultiplexSSHManager,
+        SSHManager,
+    )
 
     @wraps(func)
     @attr('skip_nomultiplex_ssh')
@@ -312,10 +559,18 @@ def skip_nomultiplex_ssh(func):
 #
 
 import os
-from datalad.support.gitrepo import GitRepo
-from datalad.support.annexrepo import AnnexRepo, FileNotInAnnexError
+
 from datalad.distribution.dataset import Dataset
-from ..utils import chpwd, getpwd
+from datalad.support.annexrepo import (
+    AnnexRepo,
+    FileNotInAnnexError,
+)
+from datalad.support.gitrepo import GitRepo
+
+from ..utils import (
+    chpwd,
+    getpwd,
+)
 
 
 def ok_clean_git(path, annex=None, index_modified=[], untracked=[]):
@@ -747,8 +1002,8 @@ class HTTPPath(object):
             # if this fails, check datalad/tests/ca/prov.sh
             # for info on deploying a datalad-root.crt
             from urllib.request import (
-                urlopen,
                 Request,
+                urlopen,
             )
             try:
                 req = Request(self.url)
@@ -1056,8 +1311,13 @@ def _get_testrepos_uris(regex, flavors):
     # we should instantiate those whenever test repos actually asked for
     # TODO: just absorb all this lazy construction within some class
     if not _TESTREPOS:
-        from .utils_testrepos import BasicAnnexTestRepo, BasicGitTestRepo, \
-            SubmoduleDataset, NestedDataset, InnerSubmodule
+        from .utils_testrepos import (
+            BasicAnnexTestRepo,
+            BasicGitTestRepo,
+            InnerSubmodule,
+            NestedDataset,
+            SubmoduleDataset,
+        )
 
         _basic_annex_test_repo = BasicAnnexTestRepo()
         _basic_git_test_repo = BasicGitTestRepo()
@@ -1643,8 +1903,8 @@ def assert_no_errors_logged(func, skip_re=None):
 
 def get_mtimes_and_digests(target_path):
     """Return digests (md5) and mtimes for all the files under target_path"""
-    from datalad.utils import find_files
     from datalad.support.digests import Digester
+    from datalad.utils import find_files
     digester = Digester(['md5'])
 
     # bother only with existing ones for this test, i.e. skip annexed files without content
@@ -2120,8 +2380,6 @@ def set_annex_version(version):
 # To be explicit, and not "loose" some tests due to typos, decided to make
 # explicit decorators for common types
 
-from nose.plugins.attrib import attr
-
 
 def integration(f):
     """Mark test as an "integration" test which generally is not needed to be run
