@@ -1,5 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -10,16 +10,19 @@
 
 __docformat__ = 'restructuredtext'
 
-from io import StringIO
 import builtins
-
+from io import StringIO
 from unittest.mock import (
     call,
     patch,
 )
 
+import pytest
+
+from datalad.ui.progressbars import progressbars
 from datalad.utils import swallow_logs
-from ...tests.utils import (
+
+from ...tests.utils_pytest import (
     assert_in,
     assert_not_in,
     assert_raises,
@@ -33,7 +36,6 @@ from ..dialog import (
     DialogUI,
     IPythonUI,
 )
-from datalad.ui.progressbars import progressbars
 
 
 def patch_input(**kwargs):
@@ -115,7 +117,11 @@ def test_hidden_doubleentry():
         gpcm.assert_has_calls([call('?: ')])
 
 
-def _test_progress_bar(backend, len, increment):
+@pytest.mark.parametrize("backend", progressbars)
+@pytest.mark.parametrize("len", [0, 4, 10, 1000])
+@pytest.mark.parametrize("increment", [True, False])
+def test_progress_bar(backend, len, increment):
+    # More of smoke testing given various lengths of fill_text
     out = StringIO()
     fill_str = ('123456890' * (len//10))[:len]
     pb = DialogUI(out).get_progressbar(
@@ -148,14 +154,6 @@ def _test_progress_bar(backend, len, increment):
             ok_endswith(output, '\r')
 
 
-def test_progress_bar():
-    # More of smoke testing given various lengths of fill_text
-    for backend in progressbars:
-        for l in 0, 4, 10, 1000:
-            for increment in True, False:
-                yield _test_progress_bar, backend, l, increment
-
-
 def test_IPythonUI():
     # largely just smoke tests to see if nothing is horribly bad
     with patch_input(return_value='a'):
@@ -179,18 +177,18 @@ def test_silent_question():
     ui = SilentConsoleLog()
     with assert_raises(RuntimeError) as cme:
         ui.question("could you help me", title="Pretty please")
-    assert_in('question: could you help me. Title: Pretty please.', str(cme.exception))
+    assert_in('question: could you help me. Title: Pretty please.', str(cme.value))
 
     with assert_raises(RuntimeError) as cme:
         ui.question("could you help me", title="Pretty please", choices=['secret1'], hidden=True)
-    assert_in('question: could you help me. Title: Pretty please.', str(cme.exception))
-    assert_not_in('secret1', str(cme.exception))
-    assert_in('not shown', str(cme.exception))
+    assert_in('question: could you help me. Title: Pretty please.', str(cme.value))
+    assert_not_in('secret1', str(cme.value))
+    assert_in('not shown', str(cme.value))
 
     # additional kwargs, no title, choices
     with assert_raises(RuntimeError) as cme:
         ui.question("q", choices=['secret1'])
-    assert_in('secret1', str(cme.exception))
+    assert_in('secret1', str(cme.value))
 
 
 @patch("datalad.log.is_interactive", lambda: False)
